/* easytag.c - 2000/04/28 */
/*
 *  EasyTAG - Viewer/Editor for MP3 Tags
 *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <id3.h>

#include "easytag.h"
#include "dirbrowser.h"
#include "misc.h"
#include "bar.h"
#include "prefs.h"
#include "config.h"
#include "undo.h"
#include "scan.h"
#include "genres.h"
#include "header.h"
#include "mylocale.h"
#include "id3tag.h"
#include "msgbox.h"

#include "../pixmaps/EasyTAG_icon.xpm"


/****************
 * Declarations *
 ****************/
guint idle_handler_id;
guint progressbar_index;  /* An integer to set the value of progress bar into the recurse fonction */

/* A variable to force to quit recursive functions (reading dirs) */
gboolean Quit_Recursion_Function = 0;
GtkWidget *QuitRecursionWindow = NULL;

/* Used to force to hide the msgbox when saving tag */
gboolean SF_HideMsgbox_Write_Tag;
/* To remember which button was pressed when saving tag */
gint SF_ButtonPressed_Write_Tag;
/* Used to force to hide the msgbox when renaming file */
gboolean SF_HideMsgbox_Rename_File;
/* To remember which button was pressed when renaming file */
gint SF_ButtonPressed_Rename_File;


/**************
 * Prototypes *
 **************/
GtkWidget *Create_Browse_Area (void);
GtkWidget *Create_File_Area (void);
GtkWidget *Create_Tag_Area (void);
GtkWidget *Create_Command_Area (void);
void Load_Genres_List (void);
void Mini_Button_Clicked (GtkObject *object);
void Insert_Only_Digit (GtkEditable *editable, const gchar *text, gint length,gint *position,gpointer data);
void Disable_Command_Buttons (void);
void Clear_Tag_Entry_Fields (void);
void Clear_Header_Fields (void);
void Parse_Genre (GtkEntry *entry, GdkEventKey *event, GtkCombo *combo);
void Spinner_Button_Activate (void);

void Write_This_Tag (void);
void Rename_File (void);
gint Save_File (gboolean multiple_files);

void Init_Load_Default_Dir (void);
void EasyTAG_Exit (void);
void Quit_MainWindow_Ok_Button (void);

guint Read_Directory_Recursively (gchar * path, gint recurse);
guint Read_Directory_Recursively_Get_Number_Of_File (gchar * path, gint recurse);
void Open_Quit_Recursion_Function_Window (void);
void Destroy_Quit_Recursion_Function_Window (void);
void Quit_Recursion_Function_Button_Pressed (void);
void Quit_Recursion_Window_Key_Press (GtkWidget *window, GdkEvent *event);



/********
 * Main *
 ********/
int main (int argc, char *argv[])
{
	GtkWidget *Table;
	GtkWidget *HBox, *VBox;
	GdkPixmap *pixmap;
	GdkBitmap *mask;


#ifdef ENABLE_NLS
	bindtextdomain("easytag",LOCALE);
	textdomain("easytag");
	/* Initialize i18n support */
	gtk_set_locale();
#endif

	/* Initialize GTK */
	gtk_init( &argc, &argv );

	/* Get home variable */
	HOME_VARIABLE = g_getenv("HOME");

	if (argc>1 && (strcmp(argv[1],"--version")==0 || strcmp(argv[1],"-v")==0) )
	{
		printf("%s %s by Jerome Couderc (compiled %s, %s)\n",APPNAME,VERSION,__TIME__,__DATE__);
		printf("Email: j.couderc@ifrance.com  Web: http://easytag.sourceforge.net\n");
		exit (0);
	}

    /* Starting message */
    g_print(_("Starting EasyTAG %s ...\n"),VERSION);
    g_print(_("Currently using id3lib version %d.%d.%d ...\n"),ID3LIB_MAJOR_VERSION,
                                                               ID3LIB_MINOR_VERSION,
                                                               ID3LIB_PATCH_VERSION);

	/* Load Config */
	Init_Config_Variables();
	Read_Config();
	/* Display_Config(); // <- for debugging */

	/* Initialization */
	Init_Mouse_Cursor();
	Init_OptionsWindow();
	Init_ScannerWindow();
	Init_Character_Translation_Table();

	/* The main window */
	MainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW (MainWindow),APPNAME" "VERSION);
	gtk_window_set_policy(GTK_WINDOW(MainWindow),FALSE,TRUE,FALSE);
	if (SET_MAIN_WINDOW_SIZE)
		gtk_window_set_default_size(GTK_WINDOW(MainWindow),MAIN_WINDOW_WIDTH,MAIN_WINDOW_HEIGHT);

	gtk_signal_connect(GTK_OBJECT(MainWindow),"delete_event",GTK_SIGNAL_FUNC(Quit_MainWindow),NULL);
	gtk_signal_connect(GTK_OBJECT(MainWindow),"destroy",GTK_SIGNAL_FUNC(Quit_MainWindow),NULL);

	/* Minimised window icon  */
	gtk_widget_realize(MainWindow);
	pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,EasyTAG_icon_xpm);
	gdk_window_set_icon(MainWindow->window,(GdkWindow *)NULL,pixmap,mask);

	Table = gtk_table_new(2,3,FALSE);
	gtk_container_add (GTK_CONTAINER(MainWindow),Table);
	gtk_widget_show(Table);

	/* Menu Bar */
	MenuArea = Create_Menu_Bar();
	gtk_table_attach(GTK_TABLE(Table),MenuArea,0,2,0,1,GTK_FILL|GTK_EXPAND,GTK_FILL,0,0);

	/* Tool Bar */
	ToolArea = Create_Tool_Bar();
	gtk_table_attach(GTK_TABLE(Table),ToolArea,0,2,1,2,GTK_FILL|GTK_EXPAND,GTK_FILL,0,0);
    
    /* The two panes: BrowserArea on the left, FileArea+TagArea on the right */
    MainWindowHPaned = gtk_hpaned_new();
	gtk_table_attach(GTK_TABLE(Table),MainWindowHPaned,0,1,2,3,GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
//    gtk_table_attach(GTK_TABLE(Table),MainWindowHPaned,0,1,1,3,GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
    gtk_paned_set_handle_size(GTK_PANED(MainWindowHPaned),8);
    gtk_paned_set_gutter_size(GTK_PANED(MainWindowHPaned),6);                       
    if (SET_PANE_HANDLE_POSITION1)
        gtk_paned_set_position(GTK_PANED(MainWindowHPaned),PANE_HANDLE_POSITION1);
	gtk_widget_show(MainWindowHPaned);

	/* Browser */
	BrowseArea = Create_Browse_Area();
    gtk_paned_pack1(GTK_PANED(MainWindowHPaned),BrowseArea,TRUE,FALSE);

    /* Vertical box for FileArea + TagArea */
	VBox = gtk_vbox_new(FALSE,0);
    gtk_paned_pack2(GTK_PANED(MainWindowHPaned),VBox,TRUE,FALSE);
	gtk_widget_show(VBox);

	/* File */
	FileArea = Create_File_Area();
	gtk_box_pack_start(GTK_BOX(VBox),FileArea,TRUE,TRUE,0);

	/* Tag */
	TagArea = Create_Tag_Area();
	gtk_box_pack_start(GTK_BOX(VBox),TagArea,TRUE,TRUE,0);

	/* Commands */
	CommandArea = Create_Command_Area();
	gtk_table_attach(GTK_TABLE(Table),CommandArea,1,2,2,3,GTK_FILL,GTK_FILL,0,0);

    /* Horizontal box for Status bar + Progress bar */
	HBox = gtk_hbox_new(FALSE,0);
	gtk_table_attach(GTK_TABLE(Table),HBox,0,2,3,4,GTK_FILL|GTK_EXPAND,GTK_FILL,0,0);
	gtk_widget_show(HBox);

	/* Status bar */
	StatusArea = Create_Status_Bar();
	gtk_box_pack_start(GTK_BOX(HBox),StatusArea,TRUE,TRUE,0);

	/* Progress bar */
	ProgressArea = Create_Progress_Bar();
	gtk_box_pack_end(GTK_BOX(HBox),ProgressArea,FALSE,FALSE,0);

	gtk_widget_show(MainWindow);

	/* Read the init directory, and as all widgets are built, if there are any mp3 in
	 * this directory, then the tag is displayed */
	/* Load the default dir when the UI is created and displayed */
	idle_handler_id = gtk_idle_add((GtkFunction)Init_Load_Default_Dir,NULL);

	/* Enter the event loop */
	gtk_main ();         
	return 0;
}


GtkWidget *Create_Browse_Area (void)
{
	GtkWidget *Frame;
	GtkWidget *Tree;

	Frame = gtk_frame_new(_("Browser"));
	gtk_container_border_width(GTK_CONTAINER(Frame),2);

	Tree = Create_Browser_Tree(MainWindow);
	gtk_container_add(GTK_CONTAINER(Frame),Tree);

	/* Don't load init dir here because Tag area hasn't been yet created!.
	 * It will be load at the end of the main function */
	//Browser_Tree_Select_Dir(DEFAULT_PATH_TO_MP3);

	gtk_widget_show(Frame);
	return Frame;
}


GtkWidget *Create_File_Area (void)
{
	GtkWidget *Frame;
	GtkWidget *VBox, *HBox;
	GtkWidget *Label;
	GtkWidget *Separator;
	GtkTooltips *Tips;


	Frame = gtk_frame_new(_("File"));
	gtk_container_border_width(GTK_CONTAINER(Frame),2);

	VBox = gtk_vbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(Frame),VBox);
	gtk_container_border_width(GTK_CONTAINER(VBox),4);

	/* Tips */
	Tips = gtk_tooltips_new();
	Set_Tips_Color(Tips);

	/* HBox for FileEntry and IconBox */
	HBox = gtk_hbox_new(FALSE,2);
	gtk_box_pack_start(GTK_BOX(VBox),HBox,TRUE,TRUE,0);

	/* File index (postion in list + list length) */
	FileIndex = gtk_label_new("?/?:");
	gtk_box_pack_start(GTK_BOX(HBox),FileIndex,FALSE,FALSE,0);

	/* File name */
	FileEntry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(HBox),FileEntry,TRUE,TRUE,0);
	gtk_entry_set_text(GTK_ENTRY(FileEntry),_("Select a directory to browse!"));
	
	/* Access status icon */
	AccessStatusIconBox = Create_Access_Icon();
	gtk_box_pack_start(GTK_BOX(HBox),AccessStatusIconBox,FALSE,FALSE,0);
	gtk_tooltips_set_tip(Tips,AccessStatusIconBox,_("Read Only File"),NULL);

	/* Create Popup Menu */
	Attach_Popup_Menu_To_Tag_Entries(GTK_ENTRY(FileEntry));


	/*
	 *  MPEG Infos
	 */
	HeaderInfosTable = gtk_table_new(3,5,FALSE);
	gtk_container_add(GTK_CONTAINER(VBox),HeaderInfosTable);
	gtk_container_border_width(GTK_CONTAINER(HeaderInfosTable),2);
	gtk_table_set_col_spacings(GTK_TABLE(HeaderInfosTable),4);

	Label = gtk_label_new(_("MPEG"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),Label,0,1,0,1);
	MpegLayerLabel = gtk_label_new(_("?, Layer ?"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),MpegLayerLabel,1,2,0,1);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(MpegLayerLabel),0,0.5);

	Label = gtk_label_new(_("Bitrate:"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),Label,0,1,1,2);
	BitrateLabel = gtk_label_new(_("? kb/s"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),BitrateLabel,1,2,1,2);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(BitrateLabel),0,0.5);

	Label = gtk_label_new(_("Freq:"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),Label,0,1,2,3);
	SamplerateLabel = gtk_label_new(_("? Hz"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),SamplerateLabel,1,2,2,3);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(SamplerateLabel),0,0.5);

	Separator = gtk_vseparator_new();
	gtk_table_attach(GTK_TABLE(HeaderInfosTable),Separator,2,3,0,4,GTK_FILL,GTK_FILL,0,0);

	Label = gtk_label_new(_("Mode:"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),Label,3,4,0,1);
	ModeLabel = gtk_label_new(_("?"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),ModeLabel,4,5,0,1);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(ModeLabel),0,0.5);

	Label = gtk_label_new(_("Size:"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),Label,3,4,1,2);
	SizeLabel = gtk_label_new(_("? kb"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),SizeLabel,4,5,1,2);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(SizeLabel),0,0.5);

	Label = gtk_label_new(_("Time:"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),Label,3,4,2,3);
	DurationLabel = gtk_label_new(_("?"));
	gtk_table_attach_defaults(GTK_TABLE(HeaderInfosTable),DurationLabel,4,5,2,3);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(DurationLabel),0,0.5);

	gtk_widget_show(Frame);
	gtk_widget_show(VBox);
	gtk_widget_show(HBox);
	gtk_widget_show(FileIndex);
	gtk_widget_show(FileEntry);
	if (SHOW_HEADER_INFO)
		gtk_widget_show_all(HeaderInfosTable);
	return Frame;
}


#include "../pixmaps/sequence_track.xpm"
GtkWidget *Create_Tag_Area (void)
{
	GtkWidget *Frame;
	GtkWidget *Separator;
	GtkWidget *Table;
	GtkWidget *Label;
	GtkWidget *Icon;
	GtkTooltips *Tips;

	Frame = gtk_frame_new(_("ID3 Tag"));
	gtk_container_border_width(GTK_CONTAINER(Frame),2);

	Table = gtk_table_new(6,8,FALSE);
	gtk_container_add(GTK_CONTAINER(Frame),Table);
	gtk_container_border_width(GTK_CONTAINER(Table),5);
	gtk_table_set_row_spacings(GTK_TABLE(Table),2);
	gtk_table_set_col_spacings(GTK_TABLE(Table),5);

	/* Tips */
	Tips = gtk_tooltips_new();
	Set_Tips_Color(Tips);

	/* Title */
	Label = gtk_label_new(_("Title:"));
	gtk_table_attach(GTK_TABLE(Table),Label,0,1,0,1,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	TitleEntry = gtk_entry_new_with_max_length(MAX_STRING_LEN);
	gtk_table_attach_defaults(GTK_TABLE(Table),TitleEntry,1,7,0,1);
	gtk_widget_set_usize(TitleEntry,200,-1);

	TitleMButton = gtk_button_new();
	gtk_widget_set_usize(TitleMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),TitleMButton,7,8,0,1,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(TitleMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,TitleMButton,_("Tag all files with this title"),NULL);

	/* Attach Popup Menu */
	Attach_Popup_Menu_To_Tag_Entries(GTK_ENTRY(TitleEntry));


	/* Artist */
	Label = gtk_label_new(_("Artist:"));
	gtk_table_attach(GTK_TABLE(Table),Label,0,1,1,2,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	ArtistEntry = gtk_entry_new_with_max_length(MAX_STRING_LEN);
	gtk_table_attach_defaults(GTK_TABLE(Table),ArtistEntry,1,7,1,2);

	ArtistMButton = gtk_button_new();
	gtk_widget_set_usize(ArtistMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),ArtistMButton,7,8,1,2,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(ArtistMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,ArtistMButton,_("Tag all files with this artist"),NULL);

	/* Attach Popup Menu */
	Attach_Popup_Menu_To_Tag_Entries(GTK_ENTRY(ArtistEntry));


	/* Album */
	Label = gtk_label_new(_("Album:"));
	gtk_table_attach(GTK_TABLE(Table),Label,0,1,2,3,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	AlbumEntry = gtk_entry_new_with_max_length(MAX_STRING_LEN);
	gtk_table_attach_defaults(GTK_TABLE(Table),AlbumEntry,1,7,2,3);

	AlbumMButton = gtk_button_new();
	gtk_widget_set_usize(AlbumMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),AlbumMButton,7,8,2,3,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(AlbumMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,AlbumMButton,_("Tag all files with this album name"),NULL);

	/* Attach Popup Menu */
	Attach_Popup_Menu_To_Tag_Entries(GTK_ENTRY(AlbumEntry));


	/* Year */
	Label = gtk_label_new(_("Year:"));
	gtk_table_attach(GTK_TABLE(Table),Label,0,1,3,4,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	YearEntry = gtk_entry_new_with_max_length(4);
	gtk_table_attach_defaults(GTK_TABLE(Table),YearEntry,1,2,3,4);
	gtk_widget_set_usize(YearEntry,45,-1);
	gtk_signal_connect(GTK_OBJECT(YearEntry),"insert_text",GTK_SIGNAL_FUNC(Insert_Only_Digit),NULL);
	gtk_signal_connect(GTK_OBJECT(YearEntry),"activate",GTK_SIGNAL_FUNC(Parse_Date),NULL);
	gtk_signal_connect(GTK_OBJECT(YearEntry),"focus-out-event",GTK_SIGNAL_FUNC(Parse_Date),NULL);

	YearMButton = gtk_button_new();
	gtk_widget_set_usize(YearMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),YearMButton,2,3,3,4,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(YearMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,YearMButton,_("Tag all files with this year"),NULL);


	/* Small vertical separator */
	Separator = gtk_vseparator_new();
	gtk_table_attach(GTK_TABLE(Table),Separator,3,4,3,4,GTK_FILL,GTK_FILL,4,2);


	/* Track */
	Label = gtk_label_new(_("Track #:"));
	gtk_table_attach(GTK_TABLE(Table),Label,5,6,3,4,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	TrackEntry = gtk_combo_new();
	gtk_table_attach_defaults(GTK_TABLE(Table),TrackEntry,6,7,3,4);
	gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(TrackEntry)->entry),MAX_STRING_LEN);
	gtk_widget_set_usize(TrackEntry,50,-1);
	gtk_signal_connect(GTK_OBJECT(GTK_ENTRY(GTK_COMBO(TrackEntry)->entry)),"insert_text",
				GTK_SIGNAL_FUNC(Insert_Only_Digit),NULL);

	TrackMButton = gtk_button_new();
	gtk_widget_set_usize(TrackMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),TrackMButton,7,8,3,4,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(TrackMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,TrackMButton,_("Tag all files with this track number"),NULL);

	TrackMButtonSequence = gtk_button_new();
	gtk_widget_set_usize(TrackMButtonSequence,10,10);
	gtk_table_attach(GTK_TABLE(Table),TrackMButtonSequence,4,5,3,4,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(TrackMButtonSequence),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,TrackMButtonSequence,_("Number all tracks sequentially. "
	                                                 "Starts at 01 in each subdirectory."), NULL);

	/* Pixmap into TrackMButtonSequence button */
	Icon = Create_Pixmap_Icon(sequence_track_xpm);
	gtk_container_add(GTK_CONTAINER(TrackMButtonSequence),Icon);
	gtk_misc_set_alignment(GTK_MISC(Icon),1,1); /* strange: values would be 0,0 !*/


	/* Genre */
	Label = gtk_label_new(_("Genre:"));
	gtk_table_attach(GTK_TABLE(Table),Label,0,1,4,5,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	GenreEntry = gtk_combo_new();
	gtk_table_attach_defaults(GTK_TABLE(Table),GenreEntry,1,7,4,5);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(GenreEntry)->entry),FALSE);
	gtk_combo_set_use_arrows_always(GTK_COMBO(GenreEntry),TRUE);
	gtk_combo_set_case_sensitive(GTK_COMBO(GenreEntry),TRUE);
	Load_Genres_List();
	gtk_signal_connect(GTK_OBJECT(GTK_ENTRY(GTK_COMBO(GenreEntry)->entry)),"key_press_event",
				(GtkSignalFunc)Parse_Genre,GTK_COMBO(GenreEntry));

	GenreMButton = gtk_button_new();
	gtk_widget_set_usize(GenreMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),GenreMButton,7,8,4,5,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(GenreMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,GenreMButton,_("Tag all files with this genre"),NULL);


	/* Comment */
	Label = gtk_label_new(_("Comment:"));
	gtk_table_attach(GTK_TABLE(Table),Label,0,1,5,6,GTK_FILL,GTK_FILL,0,0);
	gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);

	CommentEntry = gtk_entry_new_with_max_length(MAX_STRING_LEN);
	gtk_table_attach_defaults(GTK_TABLE(Table),CommentEntry,1,7,5,6);

	CommentMButton = gtk_button_new();
	gtk_widget_set_usize(CommentMButton,10,10);
	gtk_table_attach(GTK_TABLE(Table),CommentMButton,7,8,5,6,0,0,0,0);
	gtk_signal_connect(GTK_OBJECT(CommentMButton),"clicked",GTK_SIGNAL_FUNC(Mini_Button_Clicked),NULL);
	gtk_tooltips_set_tip(Tips,CommentMButton,_("Tag all files with this comment"),NULL);

	/* Attach Popup Menu */
	Attach_Popup_Menu_To_Tag_Entries(GTK_ENTRY(CommentEntry));


	gtk_widget_show_all(Frame);
	return Frame;
}


#include "../pixmaps/go.xpm"
#include "../pixmaps/next.xpm"
#include "../pixmaps/previous.xpm"
#include "../pixmaps/scan.xpm"
#include "../pixmaps/scan_all.xpm"
#include "../pixmaps/delete.xpm"
#include "../pixmaps/delete_all.xpm"
#include "../pixmaps/undo.xpm"
#include "../pixmaps/undo_all.xpm"
#include "../pixmaps/save.xpm"
#include "../pixmaps/save_all.xpm"
#include "../pixmaps/exit.xpm"
GtkWidget *Create_Command_Area (void)
{
	GtkWidget *Frame;
	GtkWidget *MainVBox, *VBox, *HBox;
	GtkTooltips *Tips;
	GtkWidget *Icon;


	Frame = gtk_frame_new(_("Commands"));
	gtk_container_border_width(GTK_CONTAINER(Frame),2);

	VBox = gtk_vbox_new(FALSE,0);	/* This VBox is used just to have buttons aligned to the bottom */
	gtk_container_add(GTK_CONTAINER(Frame),VBox);
	MainVBox = gtk_vbox_new(FALSE,4);
	gtk_box_pack_end(GTK_BOX(VBox),MainVBox,FALSE,FALSE,0);
	gtk_container_border_width(GTK_CONTAINER(MainVBox),4);

	/*
	 * Spin button
	 */
	HBox = gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(MainVBox),HBox,FALSE,FALSE,0);

	SpinButtonAdjust = (GtkAdjustment *) gtk_adjustment_new(0,0,0,1,10,10);
	SpinButton = gtk_spin_button_new(SpinButtonAdjust,1.0,0);
	gtk_box_pack_start(GTK_BOX(HBox),SpinButton,TRUE,TRUE,0);
	gtk_signal_connect(GTK_OBJECT(SpinButton),"activate",(GtkSignalFunc)Spinner_Button_Activate,NULL);
	// FIXME !!
	//gtk_signal_connect(GTK_OBJECT(SpinButton),"changed",(GtkSignalFunc)Action_Select_Nth_File,NULL);

	GoButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(go_xpm);
	gtk_container_add(GTK_CONTAINER(GoButton),Icon);
	gtk_misc_set_alignment(GTK_MISC(Icon),0.5,0.4);
	gtk_box_pack_start(GTK_BOX(HBox),GoButton,FALSE,FALSE,0);
	gtk_button_set_relief(GTK_BUTTON(GoButton),GTK_RELIEF_NONE);
	gtk_signal_connect(GTK_OBJECT(GoButton),"clicked",(GtkSignalFunc)Spinner_Button_Activate,NULL);


	/*
	 * Buttons: First, Prev, Next, Last.
	 */
	HBox = gtk_hbox_new(FALSE,3);
	gtk_box_pack_start(GTK_BOX(MainVBox),HBox,FALSE,FALSE,0);
	FirstButton = gtk_button_new();
	PrevButton  = gtk_button_new();
	NextButton  = gtk_button_new();
	LastButton  = gtk_button_new();
	gtk_widget_set_usize(FirstButton,6,-1);
	gtk_widget_set_usize(LastButton, 6,-1);
	gtk_box_pack_start(GTK_BOX(HBox),FirstButton,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(HBox),PrevButton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(HBox),NextButton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(HBox),LastButton,FALSE,FALSE,0);


	/* Pixmaps into buttons */
	Icon = Create_Pixmap_Icon(previous_xpm);
	gtk_container_add(GTK_CONTAINER(PrevButton),Icon);
	gtk_misc_set_alignment(GTK_MISC(Icon),0.4,0.5);
	Icon = Create_Pixmap_Icon(next_xpm);
	gtk_misc_set_alignment(GTK_MISC(Icon),0.5,0.5);
	gtk_container_add(GTK_CONTAINER(NextButton),Icon);

	gtk_signal_connect(GTK_OBJECT(FirstButton),"clicked",GTK_SIGNAL_FUNC(Action_Select_First_File),NULL);
	gtk_signal_connect(GTK_OBJECT(PrevButton),"clicked",GTK_SIGNAL_FUNC(Action_Select_Prev_File),NULL);
	gtk_signal_connect(GTK_OBJECT(NextButton),"clicked",GTK_SIGNAL_FUNC(Action_Select_Next_File),NULL);
	gtk_signal_connect(GTK_OBJECT(LastButton),"clicked",GTK_SIGNAL_FUNC(Action_Select_Last_File),NULL);

	/*
	 * Buttons: Scan.
	 */
	HBox = gtk_hbox_new(FALSE,3);
	gtk_box_pack_start(GTK_BOX(MainVBox),HBox,FALSE,FALSE,0);

	ScanButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(scan_xpm);
	gtk_container_add(GTK_CONTAINER(ScanButton),Icon);

	ScanAllButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(scan_all_xpm);
	gtk_container_add(GTK_CONTAINER(ScanAllButton),Icon);

	gtk_box_pack_start(GTK_BOX(HBox),ScanButton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(HBox),ScanAllButton,TRUE,TRUE,0);

	gtk_signal_connect(GTK_OBJECT(ScanButton),"clicked",GTK_SIGNAL_FUNC(Action_Scan_File),NULL);
	gtk_signal_connect(GTK_OBJECT(ScanButton),"enter",GTK_SIGNAL_FUNC(Scan_Display_Info),NULL);
	gtk_signal_connect(GTK_OBJECT(ScanAllButton),"clicked",GTK_SIGNAL_FUNC(Action_Scan_All_Files),NULL);

	/*
	 * Buttons: Remove.
	 */
	HBox = gtk_hbox_new(FALSE,3);
	gtk_box_pack_start(GTK_BOX(MainVBox),HBox,FALSE,FALSE,0);

	RemoveButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(delete_xpm);
	gtk_container_add(GTK_CONTAINER(RemoveButton),Icon);

	RemoveAllButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(delete_all_xpm);
	gtk_container_add(GTK_CONTAINER(RemoveAllButton),Icon);

	gtk_box_pack_start(GTK_BOX(HBox),RemoveButton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(HBox),RemoveAllButton,TRUE,TRUE,0);

	gtk_signal_connect(GTK_OBJECT(RemoveButton),"clicked",GTK_SIGNAL_FUNC(Action_Remove_Tag),NULL);
	gtk_signal_connect(GTK_OBJECT(RemoveAllButton),"clicked",GTK_SIGNAL_FUNC(Action_Remove_All_Tags),NULL);

	/*
	 * Buttons: Undo.
	 */
	HBox = gtk_hbox_new(FALSE,3);
	gtk_box_pack_start(GTK_BOX(MainVBox),HBox,FALSE,FALSE,0);

	UndoButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(undo_xpm);
	gtk_container_add(GTK_CONTAINER(UndoButton),Icon);

	UndoAllButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(undo_all_xpm);
	gtk_container_add(GTK_CONTAINER(UndoAllButton),Icon);

	gtk_box_pack_start(GTK_BOX(HBox),UndoButton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(HBox),UndoAllButton,TRUE,TRUE,0);

	gtk_signal_connect(GTK_OBJECT(UndoButton),"clicked",GTK_SIGNAL_FUNC(Action_Undo_File),NULL);
	gtk_signal_connect(GTK_OBJECT(UndoButton),"enter",GTK_SIGNAL_FUNC(Undo_Display_Last_Info),NULL);
	gtk_signal_connect_after(GTK_OBJECT(UndoButton),"clicked",GTK_SIGNAL_FUNC(Undo_Display_Last_Info),NULL);
	gtk_signal_connect(GTK_OBJECT(UndoAllButton),"clicked",GTK_SIGNAL_FUNC(Action_Undo_All_Files),NULL);
	//gtk_signal_connect(GTK_OBJECT(UndoAllButton),"enter",GTK_SIGNAL_FUNC(Undo_Display_Last_Info),NULL);
	gtk_signal_connect_after(GTK_OBJECT(UndoAllButton),"clicked",GTK_SIGNAL_FUNC(Undo_Display_Last_Info),NULL);

	/*
	 * Buttons: Apply/Save.
	 */
	HBox = gtk_hbox_new(FALSE,3);
	gtk_box_pack_start(GTK_BOX(MainVBox),HBox,FALSE,FALSE,0);

	ApplyButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(save_xpm);
	gtk_container_add(GTK_CONTAINER(ApplyButton),Icon);

	ApplyAllButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(save_all_xpm);
	gtk_container_add(GTK_CONTAINER(ApplyAllButton),Icon);

	gtk_box_pack_start(GTK_BOX(HBox),ApplyButton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(HBox),ApplyAllButton,TRUE,TRUE,0);

	gtk_signal_connect(GTK_OBJECT(ApplyButton),"clicked",GTK_SIGNAL_FUNC(Action_Save_File),NULL);
	gtk_signal_connect(GTK_OBJECT(ApplyAllButton),"clicked",GTK_SIGNAL_FUNC(Action_Save_All_Files),NULL);

	/*
	 * Button: Quit.
	 */
	QuitButton = gtk_button_new();
	Icon = Create_Pixmap_Icon(exit_xpm);
	gtk_container_add(GTK_CONTAINER(QuitButton),Icon);
	gtk_box_pack_start(GTK_BOX(MainVBox),QuitButton,FALSE,FALSE,0);
	gtk_signal_connect(GTK_OBJECT(QuitButton),"clicked",GTK_SIGNAL_FUNC(Quit_MainWindow),NULL);


	/* Tips */
	Tips = gtk_tooltips_new();
	gtk_tooltips_set_delay(Tips,1000);
	Set_Tips_Color(Tips);
	gtk_tooltips_set_tip(Tips,SpinButton,_("Select position of file in list, and "
		"press Enter key or GO button"),NULL);
	gtk_tooltips_set_tip(Tips,GoButton,       _("Go to this file"),NULL);
	gtk_tooltips_set_tip(Tips,FirstButton,    _("Select first file"),NULL);
	gtk_tooltips_set_tip(Tips,PrevButton,     _("Select previous file"),NULL);
	gtk_tooltips_set_tip(Tips,NextButton,     _("Select next file"),NULL);
	gtk_tooltips_set_tip(Tips,LastButton,     _("Select last file"),NULL);
	gtk_tooltips_set_tip(Tips,ScanButton,     _("Scan this file"),NULL);
	gtk_tooltips_set_tip(Tips,ScanAllButton,  _("Scan all files"),NULL);
	gtk_tooltips_set_tip(Tips,RemoveButton,   _("Remove this tag"),NULL);
	gtk_tooltips_set_tip(Tips,RemoveAllButton,_("Remove all tags"),NULL);
	gtk_tooltips_set_tip(Tips,UndoButton,     _("Undo last changes"),NULL);
	gtk_tooltips_set_tip(Tips,UndoAllButton,  _("Undo all changes"),NULL);
	gtk_tooltips_set_tip(Tips,ApplyButton,    _("Save this file"),NULL);
	gtk_tooltips_set_tip(Tips,ApplyAllButton, _("Save all files"),NULL);
	gtk_tooltips_set_tip(Tips,QuitButton,     _("Exit EasyTAG!"),NULL);

    if (SHOW_COMMAND_BUTTONS)    gtk_widget_show_all(Frame);
    else                         gtk_widget_hide_all(Frame);
    /* Select buttons to display or to hide */
    Update_Button_Visibility_Into_Command_Bar();

	return Frame;
}


/*
 * Actions when mini buttons are pressed: apply the field to all others files
 */
void Mini_Button_Clicked (GtkObject *object)
{
	GList *save_position = NULL;
	gchar *string_to_set = NULL;
	gchar *msg = NULL;


	if (!Mp3FileList) return;

	/* Save the current position in the list */
	save_position = Mp3FileList;
	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	if (object==GTK_OBJECT(TitleMButton))
	{
		/* The string to apply to all other files */
		string_to_set = g_strdup(((MP3_File *)Mp3FileList->data)->id3.title);
		/* "Rewind" list to the 1rst element */
		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
			if (string_to_set != NULL)
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.title == NULL )
					((MP3_File *)Mp3FileList->data)->id3.title = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.title,string_to_set,MAX_STRING_LEN);
			}else
			{
				/* Free data */
				if ( ((MP3_File *)Mp3FileList->data)->id3.title != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.title);
				((MP3_File *)Mp3FileList->data)->id3.title = NULL;
			}
			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
        if (string_to_set != NULL)
            msg = g_strdup_printf(_("All files tagged with title '%s'."),string_to_set);
        else
            msg = g_strdup(_("Removed title from all files."));
	}
	else if (object==GTK_OBJECT(ArtistMButton))
	{
		string_to_set = g_strdup(((MP3_File *)Mp3FileList->data)->id3.artist);
		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
			if (string_to_set != NULL)
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.artist == NULL )
					((MP3_File *)Mp3FileList->data)->id3.artist = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.artist,string_to_set,MAX_STRING_LEN);
			}else
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.artist != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.artist);
				((MP3_File *)Mp3FileList->data)->id3.artist = NULL;
			}
			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
        if (string_to_set != NULL)
            msg = g_strdup_printf(_("All files tagged with artist '%s'."),string_to_set);
        else
            msg = g_strdup(_("Removed artist from all files."));
	}
	else if (object==GTK_OBJECT(AlbumMButton))
	{
		string_to_set = g_strdup(((MP3_File *)Mp3FileList->data)->id3.album);
		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
			if (string_to_set != NULL)
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.album == NULL )
					((MP3_File *)Mp3FileList->data)->id3.album = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.album,string_to_set,MAX_STRING_LEN);
			}else
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.album != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.album);
				((MP3_File *)Mp3FileList->data)->id3.album = NULL;
			}
			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
        if (string_to_set != NULL)
            msg = g_strdup_printf(_("All files tagged with album '%s'."),string_to_set);
        else
            msg = g_strdup(_("Removed album name from all files."));
	}
	else if (object==GTK_OBJECT(YearMButton))
	{
		string_to_set = g_strdup(((MP3_File *)Mp3FileList->data)->id3.year);
		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
			if (string_to_set != NULL)
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.year == NULL )
					((MP3_File *)Mp3FileList->data)->id3.year = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.year,string_to_set,MAX_STRING_LEN);
			}else
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.year != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.year);
				((MP3_File *)Mp3FileList->data)->id3.year = NULL;
			}
			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
        if (string_to_set != NULL)
            msg = g_strdup_printf(_("All files tagged with year '%s'."),string_to_set);
        else
            msg = g_strdup(_("Removed year from all files."));
	}
	else if (object==GTK_OBJECT(TrackMButton))
	{
		string_to_set = g_strdup(((MP3_File *)Mp3FileList->data)->id3.track);
		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
			if (string_to_set != NULL)
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.track == NULL )
					((MP3_File *)Mp3FileList->data)->id3.track = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.track,string_to_set,MAX_STRING_LEN);
			}else
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.track != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.track);
				((MP3_File *)Mp3FileList->data)->id3.track = NULL;
			}
			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
		if ( string_to_set && atoi(string_to_set)>0 )
		    msg = g_strdup_printf(_("All files tagged with track '%s'."),string_to_set);
		else
		    msg = g_strdup(_("Removed track number from all files."));
	}
	/* This part doesn't set the same track number to all files, but sequence the tracks */
	else if (object==GTK_OBJECT(TrackMButtonSequence))
	{
		gchar *path = NULL;
		gchar *path1 = NULL;
		gint i = 0;

		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{       
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);

			// Restart counter when new directory
			if (path1) g_free(path1);
			path1 = g_dirname(((MP3_File *)Mp3FileList->data)->file);
			if ( path && path1 && strcmp(path,path1)!=0 )
				i = 0;
			if ( ((MP3_File *)Mp3FileList->data)->id3.track == NULL )
				((MP3_File *)Mp3FileList->data)->id3.track = g_malloc0(MAX_STRING_LEN+1);
			snprintf(((MP3_File *)Mp3FileList->data)->id3.track,MAX_STRING_LEN,"%.2d",++i);
			if (path) g_free(path);
			path = g_strdup(path1);

			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
		if (path)  g_free(path);
		if (path1) g_free(path1);
		msg = g_strdup_printf(_("All %d tracks numbered sequentially."), ListLength);
		Id3tag_Display_Tag ((MP3_File *)save_position->data);
	}
	else if (object==GTK_OBJECT(GenreMButton))
	{
		gchar *buffer = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(GenreEntry)->entry));
		Id3tag_Strip(buffer);
		string_to_set = g_strdup_printf("(%d)",Id3tag_String_To_Genre(buffer));

		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);

			if ( strlen(buffer)>0 )
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.genre == NULL )
					((MP3_File *)Mp3FileList->data)->id3.genre = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.genre,string_to_set,MAX_STRING_LEN);
			}else
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.genre != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.genre);
				((MP3_File *)Mp3FileList->data)->id3.genre = NULL;
			}

			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
        if ( strlen(buffer)>0 )
            msg = g_strdup_printf(_("All files tagged with genre '%s'."),buffer);
        else
            msg = g_strdup(_("Removed genre from all files."));
	}
	else if (object==GTK_OBJECT(CommentMButton))
	{
		string_to_set = g_strdup(((MP3_File *)Mp3FileList->data)->id3.comment);
		Mp3FileList = g_list_first(Mp3FileList);
		while (Mp3FileList)
		{	
			Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
			if (string_to_set != NULL)
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.comment == NULL )
					((MP3_File *)Mp3FileList->data)->id3.comment = g_malloc0(MAX_STRING_LEN+1);
				strncpy(((MP3_File *)Mp3FileList->data)->id3.comment,string_to_set,MAX_STRING_LEN);
			}else
			{
				if ( ((MP3_File *)Mp3FileList->data)->id3.comment != NULL )
					g_free(((MP3_File *)Mp3FileList->data)->id3.comment);
				((MP3_File *)Mp3FileList->data)->id3.comment = NULL;
			}
			Undo_Add_Data((MP3_File *)Mp3FileList->data);
			Mp3FileList = g_list_next(Mp3FileList);
		}
        if (string_to_set != NULL)
            msg = g_strdup_printf(_("All files tagged with comment '%s'."),string_to_set);
        else
            msg = g_strdup(_("Removed comment from all files."));
	}

	/* Return to the saved position in the list */
	Mp3FileList = save_position;
	if (msg)
	{
		g_print("%s\n",msg);
		Statusbar_Message(msg,TRUE);
		g_free(msg);
	}
	if (string_to_set) g_free(string_to_set);

	/* To update state of Undo button */
	Update_Command_Buttons_Sensivity();
	/* Take a snapshot to synchronized snapshot with current position in list for future changes*/
	Undo_Get_Snap_Shot((MP3_File *)Mp3FileList->data);
}


/*
 * Action when First button is selected
 */
void Action_Select_First_File (void)
{
	if (!Mp3FileList || !Mp3FileList->prev)
		return;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	ListIndex = 1;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(SpinButton),(gfloat)ListIndex);

	/* Go to the first item of the list */
	Mp3FileList = g_list_first(Mp3FileList);
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	if (GTK_WIDGET_HAS_FOCUS(FirstButton)) 
		gtk_window_set_focus(GTK_WINDOW(MainWindow),NextButton);
	Update_Command_Buttons_Sensivity();
	//Statusbar_Message(_("First file selected..."),TRUE);
}


/*
 * Action when Prev button is selected
 */
void Action_Select_Prev_File (void)
{
	if (!Mp3FileList || !Mp3FileList->prev)
		return;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	ListIndex--;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(SpinButton),(gfloat)ListIndex);

	/* Go to the prev item of the list */
	Mp3FileList = g_list_previous(Mp3FileList);
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	/* Set focus to the next button only if we reach the first when clicking 
	 * the prev button, and not when using keyboard shortcut */
	if (!Mp3FileList->prev && GTK_WIDGET_HAS_FOCUS(PrevButton)) 
		gtk_window_set_focus(GTK_WINDOW(MainWindow),NextButton);
	Update_Command_Buttons_Sensivity();
}


/*
 * Action when Next button is selected
 */
void Action_Select_Next_File (void)
{
	if (!Mp3FileList || !Mp3FileList->next)
		return;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	ListIndex++;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(SpinButton),(gfloat)ListIndex);

	/* Go to the next item of the list */
	Mp3FileList = g_list_next(Mp3FileList);
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	if (!Mp3FileList->next && GTK_WIDGET_HAS_FOCUS(NextButton)) 
		gtk_window_set_focus(GTK_WINDOW(MainWindow),PrevButton);
	Update_Command_Buttons_Sensivity();
}


/*
 * Action when Last button is selected
 */
void Action_Select_Last_File (void)
{
	if (!Mp3FileList || !Mp3FileList->next)
		return;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	ListIndex = ListLength;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(SpinButton),(gfloat)ListIndex);

	/* Go to the first item of the list */
	Mp3FileList = g_list_last(Mp3FileList);
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	if (GTK_WIDGET_HAS_FOCUS(LastButton)) 
		gtk_window_set_focus(GTK_WINDOW(MainWindow),PrevButton);
	Update_Command_Buttons_Sensivity();
	//Statusbar_Message(_("Last file selected..."),TRUE);
}


/*
 * Select the nth file in list.
 * Note: element 'n' of the list, has position 'n-1' in the list.
 */
void Action_Select_Nth_File_By_Num (gint num_item)
{
	if (!Mp3FileList || !num_item)
		return;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	/* Go to the nth item of the list */
	Mp3FileList = g_list_first(Mp3FileList);
	Mp3FileList = g_list_nth(Mp3FileList,num_item-1);
	ListIndex = num_item;
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	Update_Command_Buttons_Sensivity();
}
/*
 * Select the nth file in list. Uses the full path of the file to find it in the list.
 */
void Action_Select_Nth_File_By_Name (gchar *path)
{
	if (!Mp3FileList || !path)
		return;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	Mp3FileList = g_list_first(Mp3FileList);
	ListIndex = 0;
	while(Mp3FileList)
	{
	    ListIndex++;
	    if ( strcmp(((MP3_File *)Mp3FileList->data)->file,path)==0 )
	        break;

	    if (!Mp3FileList->next) break;
	    Mp3FileList = Mp3FileList->next;
	}
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	Update_Command_Buttons_Sensivity();
}



/*
 * Action when Scan button is selected
 */
void Action_Scan_File (void)
{
	if (!Mp3FileList) return;

	/* If Scan window is closed, then open it. If it's opened, scan tag */
	if (ScannerWindow)
	{
		Scan_Select_Mode_And_Run_Scanner();
	}else
	{
		Open_ScannerWindow();
		Statusbar_Message(_("Press the Scan button again to run the scanner."),TRUE);
	}
}


void Action_Scan_All_Files (void)
{
	gint progress_bar_index;

	if (!Mp3FileList) return;

	/* Select the first file */
	Action_Select_First_File();

	/* Initialize status bar */
	progress_bar_index = g_list_length(Mp3FileList);
	gtk_progress_configure(GTK_PROGRESS(ProgressBar),0,0,progress_bar_index);
	progress_bar_index = 0;

	/* Check if scanner window is opened */
	if(!ScannerWindow)
	{
		Open_ScannerWindow();
		Statusbar_Message(_("Select Mode and Mask, and redo the same action"),TRUE);
		return;
	}

	while (TRUE)
	{
		Action_Scan_File();

		gtk_progress_set_value(GTK_PROGRESS(ProgressBar),++progress_bar_index);
		/* Needed to refresh status bar */
		while (gtk_events_pending())
			gtk_main_iteration();

		if (!Mp3FileList->next) break;
		Action_Select_Next_File();
	}

	/* Return to the first file */
	Action_Select_First_File();

	gtk_progress_set_value(GTK_PROGRESS(ProgressBar),0);
	Statusbar_Message(_("All tags have been scanned"),TRUE);
}



/*
 * Action when Remove button is selected
 */
void Action_Remove_Tag (void)
{
	if (!Mp3FileList) return;

	Clear_Tag_Entry_Fields();
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	/* Needed to run 'Id3tag_Display_Tag' to refresh data for undo */
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

	Update_Command_Buttons_Sensivity();
	Statusbar_Message(_("Tag removed"),TRUE);
}


void Action_Remove_All_Tags (void)
{
	gint progress_bar_index;

	if (!Mp3FileList) return;

	/* Select the first file */
	Action_Select_First_File();

	/* Initialize status bar */
	progress_bar_index = g_list_length(Mp3FileList);
	gtk_progress_configure(GTK_PROGRESS(ProgressBar),0,0,progress_bar_index);
	progress_bar_index = 0;

	while (TRUE)
	{
		Id3tag_Display_Tag((MP3_File *)Mp3FileList->data);
		Action_Remove_Tag();

		gtk_progress_set_value(GTK_PROGRESS(ProgressBar),++progress_bar_index);
		/* Needed to refresh status bar */
		while (gtk_events_pending())
			gtk_main_iteration();

		if (!Mp3FileList->next) break;
		Action_Select_Next_File();
	}

	/* Return to the first file */
	Action_Select_First_File();

	gtk_progress_set_value(GTK_PROGRESS(ProgressBar),0);
	Statusbar_Message(_("All tags have been scanned"),TRUE);
}



/*
 * Action when Undo button is selected
 */
gint Action_Undo_File (void)
{
    gint state;
    
	if (!Mp3FileList || !UndoDataList) return 0;

	state = Undo_Restore_Data();
	Update_Command_Buttons_Sensivity();
	return state;
}


void Action_Undo_All_Files (void)
{
    gint state = 1;

	while(UndoDataList && state)
		state = Action_Undo_File();
}



/*
 * Save_All_Tags: Action when Apply button is selected
 * Write tags to MP3 files
 */
void Write_This_Tag (void)
{
    Statusbar_Message(_("Writing tag(s)..."),TRUE);
	if ( Id3tag_Write_Tag((MP3_File *)Mp3FileList->data)==0 )
	{
		gchar *msg;
		GtkWidget *msgbox;

		msg = g_strdup_printf(_("Can't write tag in file '%s'!\n(%s)"),
					g_basename(((MP3_File *)Mp3FileList->data)->file),
					g_strerror(errno)); 
		msgbox = msg_box_new (_("Error..."),msg,MSG_ERROR,BUTTON_OK,0);
		g_free(msg);
		msg_box_hide_check_button(MSG_BOX(msgbox));
		msg_box_run(MSG_BOX(msgbox));
		gtk_widget_destroy(msgbox);
	}else
	{
		Statusbar_Message(_("Tag(s) written"),TRUE);
	}
}
void Rename_File (void)
{
	FILE *file;
	gchar *tmpfilename = NULL;

    if ( ((MP3_File *)Mp3FileList->data)->file_new == NULL)
        return;

    Statusbar_Message(_("Renaming file(s)..."),TRUE);
	/* Check if a file already exists with the new name (detect if it's only a case change) */
	if ( (file=fopen(((MP3_File *)Mp3FileList->data)->file_new,"r+"))!=NULL
	   && strcasecmp(((MP3_File *)Mp3FileList->data)->file,((MP3_File *)Mp3FileList->data)->file_new)!=0 )
	{
		gchar *msg;
		GtkWidget *msgbox;

		fclose(file);
		msg = g_strdup_printf(_("Can't rename file \n'%s'\nbecause the following "
					"file already exists:\n'%s'"),
					g_basename(((MP3_File *)Mp3FileList->data)->file),
					g_basename(((MP3_File *)Mp3FileList->data)->file_new)); 
		msgbox = msg_box_new (_("Error..."),msg,MSG_ERROR,BUTTON_OK,0);
		g_free(msg);
		msg_box_hide_check_button(MSG_BOX(msgbox));
		msg_box_run(MSG_BOX(msgbox));
		gtk_widget_destroy(msgbox);

		Statusbar_Message(_("File(s) not renamed..."),TRUE);
		return;
	}

	/* We use two stages to rename file, to avoid problem with some system that doesn't allow to 
	 * rename the file if only the case has changed. */
	tmpfilename = g_strconcat(((MP3_File *)Mp3FileList->data)->file,".EasyTAG",NULL);
	if ( rename(((MP3_File *)Mp3FileList->data)->file,tmpfilename)!=0 )
	{
		gchar *msg;
		GtkWidget *msgbox;

		/* Renaming file to the temporary filename has failed */
		msg = g_strdup_printf(_("Can't rename file '%s'\n to \n'%s'!\n(%s)"),
					g_basename(((MP3_File *)Mp3FileList->data)->file),
					g_basename(((MP3_File *)Mp3FileList->data)->file_new),
					g_strerror(errno)); 
		msgbox = msg_box_new (_("Error..."),msg,MSG_ERROR,BUTTON_OK,0);
		g_free(msg);
		msg_box_hide_check_button(MSG_BOX(msgbox));
		msg_box_run(MSG_BOX(msgbox));
		gtk_widget_destroy(msgbox);

		Statusbar_Message(_("File(s) not renamed..."),TRUE);
		if (tmpfilename) g_free(tmpfilename);
		return;
	}

	/* Now, we rename the file to his final name */
	if ( rename(tmpfilename,((MP3_File *)Mp3FileList->data)->file_new)==0 )
	{
		/* Renaming file has succeeded */
		g_print(_("Renamed file '%s' to '%s'\n"),g_basename(((MP3_File *)Mp3FileList->data)->file),
			g_basename(((MP3_File *)Mp3FileList->data)->file_new));

		/* Update the file name into the undo list */
        Undo_Update_Data((MP3_File *)Mp3FileList->data);

		/* Update the file name data into the list */
		g_free(((MP3_File *)Mp3FileList->data)->file);
		((MP3_File *)Mp3FileList->data)->file = ((MP3_File *)Mp3FileList->data)->file_new;
		((MP3_File *)Mp3FileList->data)->file_new = NULL;

        /* Now the file was renamed, so deactivate it */
        ((MP3_File *)Mp3FileList->data)->changed &= ~FILENAME_CHANGED;

		Statusbar_Message(_("File(s) renamed..."),TRUE);
	}else
	{
		gchar *msg;
		GtkWidget *msgbox;

		/* Renaming file has failed, but we try to set the initial name */
		rename(tmpfilename,((MP3_File *)Mp3FileList->data)->file);

		msg = g_strdup_printf(_("Can't rename file '%s'\n to \n'%s'!\n(%s)"),
					g_basename(((MP3_File *)Mp3FileList->data)->file),
					g_basename(((MP3_File *)Mp3FileList->data)->file_new),
					g_strerror(errno)); 
		msgbox = msg_box_new (_("Error..."),msg,MSG_ERROR,BUTTON_OK,0);
		g_free(msg);
		msg_box_hide_check_button(MSG_BOX(msgbox));
		msg_box_run(MSG_BOX(msgbox));
		gtk_widget_destroy(msgbox);

		Statusbar_Message(_("File(s) not renamed..."),TRUE);
	}
	if (tmpfilename) g_free(tmpfilename);
}

/*
 * Saving severals files, "multiple_files": TRUE->show cancel button + check button
 *                        "hide_msgbox"   : TRUE-> don't use the msgbox
 */
gint Save_File (gboolean multiple_files)
{
	gchar *msg = NULL;
	gint stop_loop = 0;


	if (!Mp3FileList) return 0;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

	/*
	 * First part: write tag informations (artist, title,...)
	 */
	/* If tag isn't modified: doesn't write it */
	if ( (((MP3_File *)Mp3FileList->data)->changed & TAG_CHANGED)==TAG_CHANGED )
	{
		GtkWidget *msgbox = NULL;
		gint button;

		if (CONFIRM_WRITE_TAG && (!multiple_files || (multiple_files & !SF_HideMsgbox_Write_Tag)))
		{
			Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);

			msg = g_strdup_printf(_("Do you want to write the tag of file\n'%s' ?"),
				g_basename(((MP3_File *)Mp3FileList->data)->file));
			if (multiple_files)
			{
				msgbox = msg_box_new (_("Write Tag..."),msg,MSG_QUESTION,
					BUTTON_YES,BUTTON_NO,BUTTON_CANCEL,0);
			}else
			{
				msgbox = msg_box_new (_("Write Tag..."),msg,MSG_QUESTION,
					BUTTON_YES,BUTTON_NO,0);
				msg_box_hide_check_button(MSG_BOX(msgbox));
			}
			g_free(msg);
			SF_ButtonPressed_Write_Tag = button = msg_box_run(MSG_BOX(msgbox));
			/* When check button in msgbox was activated */
			if (MSG_BOX(msgbox)->check_button_state)
				SF_HideMsgbox_Write_Tag = MSG_BOX(msgbox)->check_button_state;
			gtk_widget_destroy(msgbox);
		}else
		{
			if (multiple_files && SF_HideMsgbox_Write_Tag)
				button = SF_ButtonPressed_Write_Tag;
			else
				button = BUTTON_YES;
		}

		switch(button)
		{
			case BUTTON_YES:
				Write_This_Tag();
				break;
			case BUTTON_NO:
				break;
			case BUTTON_CANCEL:
			case -1:
				stop_loop = -1;
				return stop_loop;
				break;
		}
	}


	/*
	 * Second part: rename the file
	 */
	/* Rename only if there is something to rename */
	if ( (((MP3_File *)Mp3FileList->data)->changed & FILENAME_CHANGED)==FILENAME_CHANGED )
	{
		GtkWidget *msgbox = NULL;
		gint button;

		if (CONFIRM_RENAME_FILE && (!multiple_files || (multiple_files & !SF_HideMsgbox_Rename_File)))
		{
			Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);
			msg = g_strdup_printf(_("Do you want to rename the file \n'%s'\nto \n'%s' ?"),
				g_basename(((MP3_File *)Mp3FileList->data)->file),
				g_basename(((MP3_File *)Mp3FileList->data)->file_new));
			if (multiple_files)
			{
				msgbox = msg_box_new (_("Rename File..."),msg,MSG_QUESTION,
					BUTTON_YES,BUTTON_NO,BUTTON_CANCEL,0);
			}else
			{
				msgbox = msg_box_new (_("Rename File..."),msg,MSG_QUESTION,
					BUTTON_YES,BUTTON_NO,0);
				msg_box_hide_check_button(MSG_BOX(msgbox));
			}
			g_free(msg);
			SF_ButtonPressed_Rename_File = button = msg_box_run(MSG_BOX(msgbox));
			if (MSG_BOX(msgbox)->check_button_state)
				SF_HideMsgbox_Rename_File = MSG_BOX(msgbox)->check_button_state;
			gtk_widget_destroy(msgbox);
		}else
		{
			if (multiple_files && SF_HideMsgbox_Rename_File)
				button = SF_ButtonPressed_Rename_File;
			else
				button = BUTTON_YES;
		}

		switch(button)
		{
			case BUTTON_YES:
				Rename_File();
				break;
			case BUTTON_NO:
				break;
			case BUTTON_CANCEL:
			case -1:
				stop_loop = -1;
				return stop_loop;
				break;
		}
	}

	return 1;
}

/*
 * Action_Save_File_With_Answer: Function to save single/multiple file(s).
 *  - multiple_files = TRUE : when saving files, a msgbox appears with ability 
 *                            to do the same action for all files.
 *  - multiple_files = FALSE : appears only a msgbox to ak confirmation.
 */
gint Action_Save_File_With_Answer (gboolean multiple_files)
{
    gint returned_value;
    
    Disable_Command_Buttons();
    Browser_Area_Set_Sensitive(FALSE);
    Tag_Area_Set_Sensitive(FALSE);
    File_Area_Set_Sensitive(FALSE);
    returned_value =  Save_File(multiple_files);
    Browser_Area_Set_Sensitive(TRUE);
    Tag_Area_Set_Sensitive(TRUE);
    File_Area_Set_Sensitive(TRUE);
    Update_Command_Buttons_Sensivity();
    return returned_value;
}
/*
 * Action_Save_File: Function to save a single file.
 */
void Action_Save_File (void)
{
    Action_Save_File_With_Answer(FALSE);
}



gint Action_Save_All_Files_With_Answer (void)
{
	gint progress_bar_index;
	gint saving_answer;
	GList *save_position = NULL;
	gint save_list_index;


    if (!Mp3FileList) return 0;

    /* Save the current position in the list */
    save_position = Mp3FileList;
    save_list_index = ListIndex;

	/* Save the current displayed tag */
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

    /* Rewind the list */
    Mp3FileList = g_list_first(Mp3FileList);
    ListIndex = 1;
    
    /* Initialize status bar */
    progress_bar_index = g_list_length(Mp3FileList);
    gtk_progress_configure(GTK_PROGRESS(ProgressBar),0,0,progress_bar_index);
    progress_bar_index = 0;

    /* Set to unsensitive all command buttons (except Quit button) */
    Disable_Command_Buttons();
    Browser_Area_Set_Sensitive(FALSE);
    Tag_Area_Set_Sensitive(FALSE);
    File_Area_Set_Sensitive(FALSE);

    /* Show msgbox (if needed) to ask confirmation */
    SF_HideMsgbox_Write_Tag = 0;
    SF_HideMsgbox_Rename_File = 0;
    
    while (TRUE)
    {
		Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);
		saving_answer = Save_File(TRUE);
		if (saving_answer == -1)
		{
			/* Stop saving files + reinit progress bar */
			gtk_progress_set_value(GTK_PROGRESS(ProgressBar),0);
			Statusbar_Message(_("All tags have been saved..."),TRUE);
            /* To update state of command buttons */
            Update_Command_Buttons_Sensivity();
            Browser_Area_Set_Sensitive(TRUE);
            Tag_Area_Set_Sensitive(TRUE);
            File_Area_Set_Sensitive(TRUE);
            
			return -1; /* We stop all actions */
		}

		gtk_progress_set_value(GTK_PROGRESS(ProgressBar),++progress_bar_index);
		/* Needed to refresh status bar */
		while (gtk_events_pending())
			gtk_main_iteration();

		if (!Mp3FileList->next) break;
		Mp3FileList = g_list_next(Mp3FileList);
		ListIndex++;
    }

	/* Return to the saved position in the list */
	Mp3FileList = save_position;
	ListIndex = save_list_index;
	Browser_List_Select_File(ListIndex);
	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);
	/* To update state of command buttons */
	Update_Command_Buttons_Sensivity();
    Browser_Area_Set_Sensitive(TRUE);
    Tag_Area_Set_Sensitive(TRUE);
    File_Area_Set_Sensitive(TRUE);

	gtk_progress_set_value(GTK_PROGRESS(ProgressBar),0);
	Statusbar_Message(_("All tags have been saved..."),TRUE);

	return 1;
}

void Action_Save_All_Files (void)
{
    Action_Save_All_Files_With_Answer();
}


void Spinner_Button_Activate (void)
{
    Action_Select_Nth_File_By_Num(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(SpinButton)));
}



/*
 * Load the genres list to the combo, and sorts it
 */
int Compare_Two_Genres (gchar *genre1,gchar *genre2)
{
	return strcmp(genre1,genre2);
}
void Load_Genres_List(void)
{
	GList *genrelist = NULL;
	gint i;

	if (!GenreEntry) return;

	/* Create GList */
	if (!genrelist)
    	{
		/* Add these elements for: no genre / unknown */
		genrelist = g_list_append(genrelist,"");
		genrelist = g_list_append(genrelist,"Unknown");
		/* Load others genres */
		for (i=0; i<GENRE_MAX; i++)
            genrelist = g_list_insert_sorted(genrelist,id3_genres[i],
                                             (GCompareFunc)Compare_Two_Genres);
    	}
	/* Connect the combo and the glist */
	gtk_combo_set_popdown_strings(GTK_COMBO(GenreEntry),genrelist);
}




/*
 * Complete genre field when typing 1srt character
 */
void Parse_Genre (GtkEntry *entry, GdkEventKey *event, GtkCombo *combo)
{
	GtkList *list;
	GList *glist, *glistmem;
	GtkListItem *list_item;
	GtkWidget *label;
	gchar *text;
	gchar *string;

	/* To delete the text entry */
	switch (event->keyval)
	{
		case GDK_BackSpace:
		case GDK_Delete:
		{
			gtk_entry_set_text(entry,"");
			return; 
		}
		case GDK_Return:
			return;
	}

	list = GTK_LIST(combo->list);
	glistmem = glist = list->children;
	text = gtk_entry_get_text(entry);

	if (strncasecmp(text,event->string,1)==0 )
	{
		/* The genre field begins with the same character, so diplay
		 * the next list item */ 
		/* Find the text entry in the list */
		while (glist)
		{
			list_item = (GtkListItem *)glist->data;
			label = GTK_BIN(list_item)->child;
			gtk_label_get(GTK_LABEL(label),&string);
			if (strcmp(string,text)==0)
			{
				/* Find the next entry matching */
				glist = g_list_next(glist);
				if (!glist) break;
				list_item = (GtkListItem *)glist->data;
				label = GTK_BIN(list_item)->child;
				gtk_label_get(GTK_LABEL(label),&string);

				if (strncasecmp(string,event->string,1)==0)
				{
					gtk_entry_set_text(entry,string);
					return;	
				}
			}
			glist = g_list_next(glist);
		}
		/* It was the last! */
		if (!glist)
			gdk_beep();
	}

	/* The genre field doesn't begin with the same character or it was 
	 * the last (with same char.), so diplay the first list item matching */ 
	glist = glistmem;	/* Rewind list */
	while (glist)
	{
		list_item = (GtkListItem *)glist->data;
		label = GTK_BIN(list_item)->child;
		gtk_label_get(GTK_LABEL(label),&string);
		if (strncasecmp(string,event->string,1)==0)
		{
			gtk_entry_set_text(entry,string);
			break;
		}
		glist = g_list_next(glist);
	}
	/* None matching */
	if (!glist)
		gdk_beep();
}




/*
 * Free the entire Mp3FileList list with all attached elements.
 */
void Free_MP3_File_List (void)
{
	if (Mp3FileList == NULL) return;

	Mp3FileList = g_list_first(Mp3FileList);
	while (Mp3FileList)
	{	
		/* Filename items */
		if (((MP3_File *)Mp3FileList->data)->file != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->file );
		if (((MP3_File *)Mp3FileList->data)->file_new != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->file_new );

		/* Tag items */
		if (((MP3_File *)Mp3FileList->data)->id3.title != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.title );
		if (((MP3_File *)Mp3FileList->data)->id3.artist != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.artist );
		if (((MP3_File *)Mp3FileList->data)->id3.album != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.album );
		if (((MP3_File *)Mp3FileList->data)->id3.year != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.year );
		if (((MP3_File *)Mp3FileList->data)->id3.track != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.track );
		if (((MP3_File *)Mp3FileList->data)->id3.genre != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.genre );
		if (((MP3_File *)Mp3FileList->data)->id3.comment != NULL)
			g_free( ((MP3_File *)Mp3FileList->data)->id3.comment );

		/* Free GList item data */
		g_free(Mp3FileList->data);

		/* Next item of the list */
		if (!Mp3FileList->next) break;
		Mp3FileList = g_list_next(Mp3FileList);
	}

	g_list_free(Mp3FileList);
	Mp3FileList = NULL;
}


/*
 * Scans the specified directory: and put mp3 files into a list
 */
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
void Read_Directory (gchar *path)
{
	DIR *dir;
	gchar *msg;
	guint nbrfile = 0;


    if (path == NULL) return;
    
	if ((dir=opendir(path)) == NULL)
	{
		g_print(_("Can't read directory '%s'! (%s)\n"),path,g_strerror(errno));
		return;
	}
	closedir(dir);

	ReadingDirectory = TRUE;	/* A flag to avoid to start an other reading */
	Set_Busy_Cursor();
	Statusbar_Message(_("Prepare reading of directory..."),FALSE);

	/* Initialize undo list and snap shot */
	Undo_Free_Snap_Shot_Data();
	Undo_Free_Undo_Data_List();
	
	/* Initialize mp3 list */
	Free_MP3_File_List();
	Update_Command_Buttons_Sensivity();
	
	/* Initialize browser clist */
	Browser_List_Clear();
	
	/* Clear entry boxes  */
	Clear_Tag_Entry_Fields();
	Clear_Header_Fields();
	ListIndex = ListLength = 0;
	gtk_label_set_text(GTK_LABEL(FileIndex),"?/?:");
	gtk_entry_set_text(GTK_ENTRY(FileEntry),_("Search in progress..."));

	TotalDuration = 0;
	TotalSize = 0;
	progressbar_index = 0;

	/* Open the window to quit recursion (only into recursion mode) */
	if (BROWSE_SUBDIR)
		Open_Quit_Recursion_Function_Window();

	/* Get the number of elements for progressbar */
	nbrfile = Read_Directory_Recursively_Get_Number_Of_File(path,BROWSE_SUBDIR);
	gtk_progress_configure(GTK_PROGRESS(ProgressBar),0,0,nbrfile);

	/* Read the directory recursively */
	msg = g_strdup_printf(_("Reading directory (%d items)..."),nbrfile);
	Statusbar_Message(msg,FALSE);
	g_free(msg);
	Read_Directory_Recursively(path,BROWSE_SUBDIR);

	/* Close window to quit recursion */
	Destroy_Quit_Recursion_Function_Window();
	Quit_Recursion_Function = 0;

	/* Update sensitivity of buttons and menus */
	Update_Command_Buttons_Sensivity();

	if (Mp3FileList)
	{
		/* Index to display position in list */
		ListIndex = 1;
		ListLength = g_list_length(Mp3FileList);

		/* Spinner button configuration */
		SpinButtonAdjust = (GtkAdjustment *) gtk_adjustment_new(1,1,ListLength,1,10,10);
		gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(SpinButton),SpinButtonAdjust);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(SpinButton),1.0);

        /* Load the list attached to the TrackEntry */
		Id3tag_Load_Track_List();
		
        /* Load the list of file into the browser clist widget */
        Browser_List_Load_Files();
        
		/* Display the first file */
		Mp3FileList = g_list_first(Mp3FileList);
    	Browser_List_Select_File(ListIndex);
		Display_All_Mp3_Infos(Mp3FileList->data);
		msg = g_strdup_printf(_("Found %d MP3 file(s) in this directory"),g_list_length(Mp3FileList));
	}else
	{
		/* Clear entry boxes */
		Clear_Tag_Entry_Fields();
		Clear_Header_Fields();

		SpinButtonAdjust = (GtkAdjustment *) gtk_adjustment_new(0,0,0,0,0,0);
		gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(SpinButton),SpinButtonAdjust);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(SpinButton),0.0);
		
		msg = g_strdup(_("No MP3 file in this directory!"));
		if (FileEntry)
			gtk_entry_set_text(GTK_ENTRY(FileEntry),msg);
		if (FileIndex)
			gtk_label_set_text(GTK_LABEL(FileIndex),"0/0:");
	}
	gtk_progress_set_value(GTK_PROGRESS(ProgressBar),0);
	Statusbar_Message(msg,TRUE);
	g_free (msg);
	Set_Unbusy_Cursor();
	ReadingDirectory = FALSE;
}



/*
 * Recurse the path to create a list of mp3 files. Return the number of mp3 found
 */
int Comp_Path (MP3_File *file1,MP3_File *file2)
{
	return strcmp(file1->file,file2->file);
}
guint Read_Directory_Recursively (gchar * path, gint recurse)
{
	DIR *dir;
	struct dirent *dirent;
	struct stat statbuf;
	gchar *filename;
	guint entries = 0;
	MP3_File *Mp3File;


	if ((dir = opendir(path)) != NULL)
	{
		while ((dirent = readdir(dir)) != NULL)
		{
			if (Quit_Recursion_Function == 1)
				return entries;

			if (dirent->d_name[0] != '.')
			{
				if (path[strlen(path)-1]!='/')
					filename = g_strconcat(path,"/",dirent->d_name,NULL);
				else
					filename = g_strconcat(path,dirent->d_name,NULL);

				lstat(filename, &statbuf);
				if (S_ISDIR(statbuf.st_mode))
				{
					if (recurse)
					{
						guint i;
						i = Read_Directory_Recursively(filename,recurse);
						entries += i;
					}
				}else if (S_ISREG(statbuf.st_mode) && Check_If_Mp3_By_Extension(filename))
				{
					Mp3File = g_malloc0(sizeof(MP3_File));
					Mp3File->file = g_strdup(filename);
					Mp3File->file_new = NULL;
					Id3tag_Read_Tag(Mp3File);
					Header_Read_Mp3(Mp3File);
					Mp3FileList = g_list_insert_sorted(Mp3FileList,Mp3File,(GCompareFunc)Comp_Path);
					entries++;
				}
				g_free(filename);
				gtk_progress_set_value(GTK_PROGRESS(ProgressBar),++progressbar_index);
				while (gtk_events_pending())
					gtk_main_iteration();
			}
		}
		closedir(dir);
	}
	return entries;
}


/*
 * Return the number of files into the directory, browsed recursively 
 * or not, just to initialize progressbar.
 */
guint Read_Directory_Recursively_Get_Number_Of_File (gchar * path, gint recurse)
{
	DIR *dir;
	struct dirent *dirent;
	struct stat statbuf;
	gchar *filename;
	guint entries = 0;


	if ((dir = opendir(path)) != NULL)
	{
		while ((dirent = readdir(dir)) != NULL)
		{
			if (Quit_Recursion_Function == 1)
				return entries;

			if (dirent->d_name[0] != '.')
			{
				if (path[strlen(path)-1]!='/')
					filename = g_strconcat(path,"/",dirent->d_name,NULL);
				else
					filename = g_strconcat(path,dirent->d_name,NULL);

				lstat(filename, &statbuf);
				if (S_ISDIR(statbuf.st_mode))
				{
					if (recurse)
					{
						guint i;
						i = Read_Directory_Recursively_Get_Number_Of_File
							(filename,recurse);
						entries += i;
					}
				}else
				{
					entries++;
				}
				g_free(filename);
				/* Just to not block X events */
				while (gtk_events_pending())
					gtk_main_iteration();
			}
		}
		closedir(dir);
	}
	return entries;
}
/*
 * Window with the 'STOP' button to stop recursion when reading directories
 */
void Open_Quit_Recursion_Function_Window (void)
{
	GtkWidget *button;
	GtkWidget *frame;
	gint mw_x, mw_y;
	gint ba_x, ba_y;


	if (QuitRecursionWindow != NULL) 
		return;
	QuitRecursionWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(QuitRecursionWindow),_("Stop"));
	gtk_window_set_transient_for(GTK_WINDOW(QuitRecursionWindow),GTK_WINDOW(MainWindow));
	//gtk_window_set_policy(GTK_WINDOW(QuitRecursionWindow),FALSE,FALSE,TRUE);
	gtk_signal_connect(GTK_OBJECT(QuitRecursionWindow),"destroy",
		(GtkSignalFunc)Quit_Recursion_Function_Button_Pressed,NULL);
	gtk_signal_connect(GTK_OBJECT(QuitRecursionWindow),"delete_event",
		(GtkSignalFunc)Quit_Recursion_Function_Button_Pressed,NULL);
	gtk_signal_connect(GTK_OBJECT(QuitRecursionWindow),"key_press_event",
        (GtkSignalFunc)Quit_Recursion_Window_Key_Press,NULL);

	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(QuitRecursionWindow),frame);
	gtk_container_set_border_width(GTK_CONTAINER(frame),2);

	/* Button to stop... */
	button = gtk_button_new_with_label(_("  STOP  "));
	gtk_container_set_border_width(GTK_CONTAINER(button),8);
	gtk_container_add(GTK_CONTAINER(frame),button);
	gtk_signal_connect(GTK_OBJECT(button),"clicked",
        (GtkSignalFunc)Quit_Recursion_Function_Button_Pressed,NULL);

	gtk_widget_show_all(QuitRecursionWindow);

    /* Note : the window must be shown before to move it */
	gdk_window_get_position(MainWindow->window,&mw_x,&mw_y);
	gdk_window_get_position(BrowseArea->window,&ba_x,&ba_y);
    gtk_window_reposition(GTK_WINDOW(QuitRecursionWindow),
		mw_x + ba_x + (BrowseArea->allocation.width/2)  - (QuitRecursionWindow->allocation.width)/2,
		mw_y + ba_y + (BrowseArea->allocation.height/2) - (QuitRecursionWindow->allocation.height)/2);
}
void Destroy_Quit_Recursion_Function_Window (void)
{
	if (QuitRecursionWindow)
	{
		gtk_widget_destroy(QuitRecursionWindow);
		QuitRecursionWindow = (GtkWidget *)NULL;
		/*Statusbar_Message(_("Recursive file search interrupted."),FALSE);*/
	}
}
void Quit_Recursion_Function_Button_Pressed (void)
{
	Quit_Recursion_Function = 1;
	Destroy_Quit_Recursion_Function_Window();
}
void Quit_Recursion_Window_Key_Press (GtkWidget *window, GdkEvent *event)
{
	GdkEventKey *kevent;

	if (event && event->type == GDK_KEY_PRESS)
	{
		kevent = (GdkEventKey *)event;
		switch(kevent->keyval)
		{
			case GDK_Escape:	
				Destroy_Quit_Recursion_Function_Window();
				break;
		}
    }
}


/*
 * Display informations about the mp3 file: Header + Tag
 */
void Display_All_Mp3_Infos (MP3_File *mp3file)
{
	gchar *msg;

	if (!Mp3FileList || !mp3file) return;

	Id3tag_Display_Tag(mp3file);
	Header_Display(mp3file);
	msg = g_strdup_printf(_("File: '%s'"),mp3file->file);
	Statusbar_Message(msg,FALSE);
	g_free(msg);
}



/*
 * Update_Command_Buttons_Sensivity: Set to sensitive/unsensitive the state of each button into
 * the commands area and menu items in function of state of the GList Mp3FileList
 */
void item_factory_set_sensitive (guint action, gboolean sensitive)
{
	GtkWidget *widget;

	widget = gtk_item_factory_get_widget_by_action(ItemFactory,action);
	if (widget) gtk_widget_set_sensitive(widget,sensitive);
}

void Update_Command_Buttons_Sensivity (void)
{
	if (!Mp3FileList)
	{
		/* No mp3 file found */
		
		/* Command buttons */
		gtk_widget_set_sensitive(SpinButton,FALSE);
		gtk_widget_set_sensitive(GoButton,FALSE);
		gtk_widget_set_sensitive(FirstButton,FALSE);
		gtk_widget_set_sensitive(PrevButton,FALSE);
		gtk_widget_set_sensitive(NextButton,FALSE);
		gtk_widget_set_sensitive(LastButton,FALSE);
		gtk_widget_set_sensitive(ScanButton,FALSE);
		gtk_widget_set_sensitive(RemoveButton,FALSE);
		gtk_widget_set_sensitive(UndoButton,FALSE);
		gtk_widget_set_sensitive(ApplyButton,FALSE);
		gtk_widget_set_sensitive(ScanAllButton,FALSE);
		gtk_widget_set_sensitive(RemoveAllButton,FALSE);
		gtk_widget_set_sensitive(UndoAllButton,FALSE);
		gtk_widget_set_sensitive(ApplyAllButton,FALSE);
		/* Tool bar buttons */
//		gtk_widget_set_sensitive(TBSpinButton,FALSE);
//		gtk_widget_set_sensitive(TBGoButton,FALSE);
		gtk_widget_set_sensitive(TBFirstButton,FALSE);
		gtk_widget_set_sensitive(TBPrevButton,FALSE);
		gtk_widget_set_sensitive(TBNextButton,FALSE);
		gtk_widget_set_sensitive(TBLastButton,FALSE);
		gtk_widget_set_sensitive(TBScanButton,FALSE);
		gtk_widget_set_sensitive(TBRemoveButton,FALSE);
		gtk_widget_set_sensitive(TBUndoButton,FALSE);
		gtk_widget_set_sensitive(TBApplyButton,FALSE);
		gtk_widget_set_sensitive(TBScanAllButton,FALSE);
		gtk_widget_set_sensitive(TBRemoveAllButton,FALSE);
		gtk_widget_set_sensitive(TBUndoAllButton,FALSE);
		gtk_widget_set_sensitive(TBApplyAllButton,FALSE);
		/* Commands into menu */
		item_factory_set_sensitive(PREV,FALSE);
		item_factory_set_sensitive(NEXT,FALSE);
		item_factory_set_sensitive(FIRST,FALSE);
		item_factory_set_sensitive(LAST,FALSE);
		item_factory_set_sensitive(SCAN,FALSE);
		item_factory_set_sensitive(REMOVE,FALSE);
		item_factory_set_sensitive(UNDO,FALSE);
		item_factory_set_sensitive(APPLY,FALSE);
		item_factory_set_sensitive(SCAN_ALL,FALSE);
		item_factory_set_sensitive(REMOVE_ALL,FALSE);
		item_factory_set_sensitive(UNDO_ALL,FALSE);
		item_factory_set_sensitive(APPLY_ALL,FALSE);
		/* File and Tag frames */
		gtk_widget_set_sensitive(GTK_BIN(FileArea)->child,FALSE);
		gtk_widget_set_sensitive(GTK_BIN(TagArea)->child,FALSE);
		return;
	}else
	{
		/* File and Tag frames */
		gtk_widget_set_sensitive(GTK_BIN(FileArea)->child,TRUE);
		gtk_widget_set_sensitive(GTK_BIN(TagArea)->child,TRUE);
		/* Command buttons */
		gtk_widget_set_sensitive(SpinButton,TRUE);
		gtk_widget_set_sensitive(GoButton,TRUE);
		gtk_widget_set_sensitive(ScanButton,TRUE);
		gtk_widget_set_sensitive(RemoveButton,TRUE);
		gtk_widget_set_sensitive(ApplyButton,TRUE);
		gtk_widget_set_sensitive(ScanAllButton,TRUE);
		gtk_widget_set_sensitive(RemoveAllButton,TRUE);
		gtk_widget_set_sensitive(ApplyAllButton,TRUE);
		/* Tool bar buttons */
//		gtk_widget_set_sensitive(TBSpinButton,TRUE);
//		gtk_widget_set_sensitive(TBGoButton,TRUE);
		gtk_widget_set_sensitive(TBScanButton,TRUE);
		gtk_widget_set_sensitive(TBRemoveButton,TRUE);
		gtk_widget_set_sensitive(TBApplyButton,TRUE);
		gtk_widget_set_sensitive(TBScanAllButton,TRUE);
		gtk_widget_set_sensitive(TBRemoveAllButton,TRUE);
		gtk_widget_set_sensitive(TBApplyAllButton,TRUE);
		/* Commands into menu */
		item_factory_set_sensitive(SCAN,TRUE);
		item_factory_set_sensitive(REMOVE,TRUE);
		item_factory_set_sensitive(APPLY,TRUE);
		item_factory_set_sensitive(SCAN_ALL,TRUE);
		item_factory_set_sensitive(REMOVE_ALL,TRUE);
		item_factory_set_sensitive(APPLY_ALL,TRUE);
        /* Enable undo buttons if there is data into undolist */
		if (UndoDataList)
		{
			gtk_widget_set_sensitive(UndoButton,TRUE);
			gtk_widget_set_sensitive(UndoAllButton,TRUE);
			gtk_widget_set_sensitive(TBUndoButton,TRUE);
			gtk_widget_set_sensitive(TBUndoAllButton,TRUE);
			item_factory_set_sensitive(UNDO,TRUE);
			item_factory_set_sensitive(UNDO_ALL,TRUE);
		}else
		{
			gtk_widget_set_sensitive(UndoButton,FALSE);
			gtk_widget_set_sensitive(UndoAllButton,FALSE);
			gtk_widget_set_sensitive(TBUndoButton,FALSE);
			gtk_widget_set_sensitive(TBUndoAllButton,FALSE);
			item_factory_set_sensitive(UNDO,FALSE);
			item_factory_set_sensitive(UNDO_ALL,FALSE);
		}
	}

	if (!Mp3FileList->prev)	/* Is it the 1rst item ? */
	{
		gtk_widget_set_sensitive(FirstButton,FALSE);
		gtk_widget_set_sensitive(PrevButton,FALSE);
		gtk_widget_set_sensitive(TBFirstButton,FALSE);
		gtk_widget_set_sensitive(TBPrevButton,FALSE);
		item_factory_set_sensitive(PREV,FALSE);
		item_factory_set_sensitive(FIRST,FALSE);
	}else
	{
		gtk_widget_set_sensitive(FirstButton,TRUE);
		gtk_widget_set_sensitive(PrevButton,TRUE);
		gtk_widget_set_sensitive(TBFirstButton,TRUE);
		gtk_widget_set_sensitive(TBPrevButton,TRUE);
		item_factory_set_sensitive(PREV,TRUE);
		item_factory_set_sensitive(FIRST,TRUE);
	}
	if (!Mp3FileList->next)	/* Is it the last item ? */
	{
		gtk_widget_set_sensitive(NextButton,FALSE);
		gtk_widget_set_sensitive(LastButton,FALSE);
		gtk_widget_set_sensitive(TBNextButton,FALSE);
		gtk_widget_set_sensitive(TBLastButton,FALSE);
		item_factory_set_sensitive(NEXT,FALSE);
		item_factory_set_sensitive(LAST,FALSE);
	}else
	{
		gtk_widget_set_sensitive(NextButton,TRUE);
		gtk_widget_set_sensitive(LastButton,TRUE);
		gtk_widget_set_sensitive(TBNextButton,TRUE);
		gtk_widget_set_sensitive(TBLastButton,TRUE);
		item_factory_set_sensitive(NEXT,TRUE);
		item_factory_set_sensitive(LAST,TRUE);
	}
}

/*
 * Just to disable buttons when we are saving files (do not disable Quit button)
 */
void Disable_Command_Buttons (void)
{
	gtk_widget_set_sensitive(SpinButton,FALSE);
	gtk_widget_set_sensitive(GoButton,FALSE);
    gtk_widget_set_sensitive(FirstButton,FALSE);
    gtk_widget_set_sensitive(PrevButton,FALSE);
    gtk_widget_set_sensitive(NextButton,FALSE);
    gtk_widget_set_sensitive(LastButton,FALSE);
    gtk_widget_set_sensitive(ScanButton,FALSE);
    gtk_widget_set_sensitive(RemoveButton,FALSE);
    gtk_widget_set_sensitive(UndoButton,FALSE);
    gtk_widget_set_sensitive(ApplyButton,FALSE);
    gtk_widget_set_sensitive(ScanAllButton,FALSE);
    gtk_widget_set_sensitive(RemoveAllButton,FALSE);
    gtk_widget_set_sensitive(UndoAllButton,FALSE);
    gtk_widget_set_sensitive(ApplyAllButton,FALSE);
    gtk_widget_set_sensitive(TBFirstButton,FALSE);
    gtk_widget_set_sensitive(TBPrevButton,FALSE);
    gtk_widget_set_sensitive(TBNextButton,FALSE);
    gtk_widget_set_sensitive(TBLastButton,FALSE);
    gtk_widget_set_sensitive(TBScanButton,FALSE);
    gtk_widget_set_sensitive(TBRemoveButton,FALSE);
    gtk_widget_set_sensitive(TBUndoButton,FALSE);
    gtk_widget_set_sensitive(TBApplyButton,FALSE);
    gtk_widget_set_sensitive(TBScanAllButton,FALSE);
    gtk_widget_set_sensitive(TBRemoveAllButton,FALSE);
    gtk_widget_set_sensitive(TBUndoAllButton,FALSE);
    gtk_widget_set_sensitive(TBApplyAllButton,FALSE);
    item_factory_set_sensitive(PREV,FALSE);
    item_factory_set_sensitive(NEXT,FALSE);
    item_factory_set_sensitive(FIRST,FALSE);
    item_factory_set_sensitive(LAST,FALSE);
    item_factory_set_sensitive(SCAN,FALSE);
    item_factory_set_sensitive(REMOVE,FALSE);
    item_factory_set_sensitive(UNDO,FALSE);
    item_factory_set_sensitive(APPLY,FALSE);
    item_factory_set_sensitive(SCAN_ALL,FALSE);
    item_factory_set_sensitive(REMOVE_ALL,FALSE);
    item_factory_set_sensitive(UNDO_ALL,FALSE);
    item_factory_set_sensitive(APPLY_ALL,FALSE);
}

/*
 * Disable (FALSE) / Enable (TRUE) all user widgets in the tag area
 */
void Tag_Area_Set_Sensitive (gboolean activate)
{
    /* TAG Area */
    gtk_widget_set_sensitive(GTK_WIDGET(TitleEntry),  activate);
    gtk_widget_set_sensitive(GTK_WIDGET(ArtistEntry), activate);
    gtk_widget_set_sensitive(GTK_WIDGET(AlbumEntry),  activate);
    gtk_widget_set_sensitive(GTK_WIDGET(YearEntry),   activate);
    gtk_widget_set_sensitive(GTK_WIDGET(TrackEntry),  activate);
    gtk_widget_set_sensitive(GTK_WIDGET(CommentEntry),activate);
    gtk_widget_set_sensitive(GTK_WIDGET(GenreEntry),  activate);
    /* Mini buttons */
    gtk_widget_set_sensitive(GTK_WIDGET(TitleMButton),  activate);
    gtk_widget_set_sensitive(GTK_WIDGET(ArtistMButton), activate);
    gtk_widget_set_sensitive(GTK_WIDGET(AlbumMButton),  activate);
    gtk_widget_set_sensitive(GTK_WIDGET(YearMButton),   activate);
    gtk_widget_set_sensitive(GTK_WIDGET(TrackMButton),  activate);
    gtk_widget_set_sensitive(GTK_WIDGET(TrackMButtonSequence),activate);
    gtk_widget_set_sensitive(GTK_WIDGET(CommentMButton),activate);
    gtk_widget_set_sensitive(GTK_WIDGET(GenreMButton),  activate);
}

/*
 * Disable (FALSE) / Enable (TRUE) all user widgets in the file area
 */
void File_Area_Set_Sensitive (gboolean activate)
{
    gtk_widget_set_sensitive(GTK_WIDGET(FileEntry),activate);
}

/*
 * Clear the entries of tag
 */
void Clear_Tag_Entry_Fields (void)
{
	if (!TitleEntry) return;

	gtk_entry_set_text(GTK_ENTRY(TitleEntry),"");
	gtk_entry_set_text(GTK_ENTRY(ArtistEntry),"");
	gtk_entry_set_text(GTK_ENTRY(AlbumEntry),"");
	gtk_entry_set_text(GTK_ENTRY(YearEntry),"");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(TrackEntry)->entry),"");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(GenreEntry)->entry),"");
	gtk_entry_set_text(GTK_ENTRY(CommentEntry),"");
}


/*
 * Clear the header informations
 */
void Clear_Header_Fields (void)
{
	if (!MpegLayerLabel) return;

	gtk_label_set_text(GTK_LABEL(MpegLayerLabel),_("?, Layer ?"));
	gtk_label_set_text(GTK_LABEL(BitrateLabel),_("? kb/s"));
	gtk_label_set_text(GTK_LABEL(SamplerateLabel),_("? Hz"));
	gtk_label_set_text(GTK_LABEL(SizeLabel),_("?"));
	gtk_label_set_text(GTK_LABEL(ModeLabel),_("?"));
	gtk_label_set_text(GTK_LABEL(DurationLabel),_("?"));
}




/*
 * Run Xmms with loading current dir
 */
#include <stdlib.h>
void Run_Xmms (void)
{
	gchar *command;
	pid_t pid;
	gint sysflag;

	pid = fork();
	switch(pid)
	{
		case -1:
			g_warning(_("Can't fork another process!\n"));
			break;
		case 0:
			command = g_strdup_printf("xmms -p \"%s\" &",Get_Current_Path());
			sysflag = system(command);
			if ( (sysflag == -1) && (sysflag == 127) )
			{
				GtkWidget *msgbox;

				msgbox = msg_box_new (_("Error..."),_("Can't start XMMS!"),
					MSG_ERROR,BUTTON_OK,0);
				msg_box_hide_check_button(MSG_BOX(msgbox));
				msg_box_run(MSG_BOX(msgbox));
				gtk_widget_destroy(msgbox);
			}else
			{
				Statusbar_Message(_("XMMS is starting..."),TRUE);
			} ;
			g_free(command);
			_exit(1);	/* Must use _exit instead of exit */
			break;
		default:
			break;
	}
}




/*
 * Load default dir when the user interface is displayed 
 * completely to avoid bad visualization effect.
 */
void Init_Load_Default_Dir (void)
{
	Browser_Tree_Select_Dir(DEFAULT_PATH_TO_MP3);
	gtk_idle_remove(idle_handler_id);
}




/*
 * Entry_Popup_Menu_Handler: show the popup menu when the third mouse button is pressed.
 */
gboolean Entry_Popup_Menu_Handler (GtkMenu *menu, GdkEventButton *event)
{
	if (event && (event->type==GDK_BUTTON_PRESS) && (event->button==3))
	{
		gtk_menu_popup(menu,NULL,NULL,NULL,NULL,event->button,event->time);
		return TRUE;
	}
	return FALSE;
}

void Convert_P20_And_Undescore_Into_Spaces (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Convert_Underscore_Into_Space(text);
		Scan_Convert_P20_Into_Space(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_Space_Into_Undescore (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Convert_Space_Into_Undescore(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_All_Uppercase (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
        //g_print("s: %d\n",GTK_EDITABLE(GTK_ENTRY(entry))->selection_start_pos);
        //g_print("e: %d\n",GTK_EDITABLE(GTK_ENTRY(entry))->selection_end_pos);
		Scan_Process_Fields_All_Uppercase(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_All_Downcase (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Process_Fields_All_Downcase(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_Letter_Uppercase (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Process_Fields_Letter_Uppercase(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_First_Letters_Uppercase (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Process_Fields_First_Letters_Uppercase(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_Remove_Space (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Process_Fields_Remove_Space(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}
void Convert_Insert_Space (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		/* Memory must be allocated cause size of string may increase */
		gchar *text = g_malloc(MAX_STRING_LEN);
		strcpy(text,gtk_entry_get_text(GTK_ENTRY(entry)));
		Scan_Process_Fields_Insert_Space(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
		g_free(text);
	}
}
void Convert_Only_One_Space (GtkWidget *entry)
{
	if (GTK_IS_ENTRY(entry))
	{
		gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
		Scan_Process_Fields_Keep_One_Space(text);
		gtk_entry_set_text(GTK_ENTRY(entry),text);
	}
}

/*
 * Popup menu attached to all entries of tag + filename + rename combobox.
 * Displayed when pressing the right mouse button and contains functions to process ths strings.
 */
void Attach_Popup_Menu_To_Tag_Entries (GtkEntry *entry)
{
	GtkWidget *PopupMenu;
	GtkWidget *MenuItem;


	PopupMenu = gtk_menu_new();
	gtk_signal_connect_object(GTK_OBJECT(entry),"button_press_event",
		(GtkSignalFunc)Entry_Popup_Menu_Handler,GTK_OBJECT(PopupMenu));

    /* Menu items */
	MenuItem = gtk_menu_item_new_with_label(_("Convert '_' and '%20' to spaces"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_P20_And_Undescore_Into_Spaces,GTK_OBJECT(entry));

	MenuItem = gtk_menu_item_new_with_label(_("Convert ' ' to '_'"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_Space_Into_Undescore,GTK_OBJECT(entry));

    /* Separator */
	MenuItem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);

	MenuItem = gtk_menu_item_new_with_label(_("All uppercase"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_All_Uppercase,GTK_OBJECT(entry));

	MenuItem = gtk_menu_item_new_with_label(_("All downcase"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_All_Downcase,GTK_OBJECT(entry));

	MenuItem = gtk_menu_item_new_with_label(_("First letter uppercase"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_Letter_Uppercase,GTK_OBJECT(entry));

	MenuItem = gtk_menu_item_new_with_label(_("First letter uppercase of each word"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_First_Letters_Uppercase,GTK_OBJECT(entry));

    /* Separator */
	MenuItem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);

	MenuItem = gtk_menu_item_new_with_label(_("Remove spaces"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_Remove_Space,GTK_OBJECT(entry));

	MenuItem = gtk_menu_item_new_with_label(_("Insert space before uppercase letter"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_Insert_Space,GTK_OBJECT(entry));

	MenuItem = gtk_menu_item_new_with_label(_("Remove duplicate spaces or underscores"));
	gtk_menu_append(GTK_MENU(PopupMenu),MenuItem);
	gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",
		(GtkSignalFunc)Convert_Only_One_Space,GTK_OBJECT(entry));

	gtk_widget_show_all(PopupMenu);
}



/*
 * Returns 1 if all files have been saved.
 * Returns 0 if some changes haven't been saved
 */
gboolean Check_If_All_Files_Are_Saved (void)
{
	gboolean some_changes_to_save = 1; // 0: not changed, 1: was changed
	GList *tmplist;

	/* Check if some file haven't been saved, if didn't nochange=0 */
	if (Mp3FileList)
	{
		/* If you change the displayed tag and quit immediately */
		Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);
		tmplist = g_list_first(Mp3FileList);
		while (tmplist && some_changes_to_save)
		{
			/* Check if tags have changed */
			if ( (((MP3_File *)tmplist->data)->changed & TAG_CHANGED)==TAG_CHANGED )	//Some changes...
				some_changes_to_save &= 0;

			/* Check if file name has changed */
			if ( (((MP3_File *)tmplist->data)->changed & FILENAME_CHANGED)==FILENAME_CHANGED )
				some_changes_to_save &= 0;

			if (!tmplist->next) break;
			tmplist = g_list_next(tmplist);
		}
	}

	return some_changes_to_save;
}





/*
 * Exit the program
 */
void EasyTAG_Exit (void)
{
	g_print(_("EasyTAG: Normal exit.\n"));
	gtk_main_quit();
	exit(0);
}

void Quit_MainWindow_Confirmed (void)
{
	/* Save config if wanted */
	if (SAVE_CONFIG_BEFORE_EXIT)
	{
		Apply_Changes(); /* Can be replaced by Save_Changes() */
		Save_Config_To_File();
	}
	EasyTAG_Exit();
}

void Quit_MainWindow_Save_And_Quit (void)
{
	/* Save modified tags */
	if (Action_Save_All_Files_With_Answer() == -1) return;
	Quit_MainWindow_Confirmed();
}

void Quit_MainWindow (void)
{
	GtkWidget *msgbox;
	gint button;
	GList *list;


    /* Save combobox history lists before to exit */
    list = gtk_object_get_data(GTK_OBJECT(BrowserEntry),"History");
    Save_Path_Entry_List(list);

    /* Exit ? */
	if(CONFIRM_BEFORE_EXIT) 
	{
		if (Check_If_All_Files_Are_Saved()==0)
		{
			/* Some files haven't been saved */
			msgbox = msg_box_new (_("Confirm..."),_("Some files have been modified but not "
				"saved...\nDo you want to save them before to exit the program?"),
				MSG_QUESTION,BUTTON_YES,BUTTON_NO,BUTTON_CANCEL,0);
			msg_box_hide_check_button(MSG_BOX(msgbox));
			button = msg_box_run(MSG_BOX(msgbox));
			gtk_widget_destroy(msgbox);
			switch(button)
			{
				case BUTTON_YES:
					Quit_MainWindow_Save_And_Quit();
					break;
				case BUTTON_NO:
					Quit_MainWindow_Confirmed();
					break;
				case BUTTON_CANCEL:
				case -1:
					return;
			}
		} else
		{
			msgbox = msg_box_new (_("Confirm..."),_(" Do you really want to exit the program? "),
				MSG_QUESTION,BUTTON_YES,BUTTON_NO,0);
			msg_box_hide_check_button(MSG_BOX(msgbox));
			button = msg_box_run(MSG_BOX(msgbox));
			gtk_widget_destroy(msgbox);
			switch(button)
			{
				case BUTTON_YES:
					Quit_MainWindow_Confirmed();
					break;
				case BUTTON_NO:
				case -1:
					return;
					break;
			}
		}
	}else if (Check_If_All_Files_Are_Saved()==0)
	{
		/* Some files aren't saved */
		msgbox = msg_box_new (_("Confirm..."),_("Some files have been modified but not "
			"saved...\nDo you want to save them before exiting the program?"),
			MSG_QUESTION,BUTTON_YES,BUTTON_NO,BUTTON_CANCEL,0);
		msg_box_hide_check_button(MSG_BOX(msgbox));
		button = msg_box_run(MSG_BOX(msgbox));
		gtk_widget_destroy(msgbox);
		switch(button)
		{
			case BUTTON_YES:
				Quit_MainWindow_Save_And_Quit();
				break;
			case BUTTON_NO:
                Quit_MainWindow_Confirmed();
				break;
			case BUTTON_CANCEL:
			case -1:
				return;
		}
	}else
	{
		Quit_MainWindow_Confirmed();
	}
}
