/* misc.c - 2000/06/28 */
/*
 *  EasyTAG - Viewer/Editor for MP3 Tags
 *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>

#include "misc.h"
#include "easytag.h"
#include "msgbox.h"
#include "id3tag.h"
#include "dirbrowser.h"
#include "config.h"
#include "mylocale.h"


/***************
 * Declaration *
 ***************/

/* For translation tables */
static unsigned char trans_serv2user[256];
static unsigned char trans_user2serv[256];


/**************
 * Prototypes *
 **************/


/*************
 * Functions *
 *************/


/******************************
 * Functions managing pixmaps *
 ******************************/
/*
 * Buttons creation with pixmap
 */
#include "../pixmaps/ok.xpm"
#include "../pixmaps/yes.xpm"
#include "../pixmaps/no.xpm"
#include "../pixmaps/apply.xpm"
#include "../pixmaps/save.xpm"
#include "../pixmaps/cancel.xpm"
#include "../pixmaps/close.xpm"
GtkWidget *Create_Button_With_Pixmap (guint button_type)
{
	GtkWidget *Button;
	GtkWidget *HBox;
	GtkWidget *Label;
	GtkWidget *PixmapIcon;
	GdkPixmap *pixmap;
	GdkBitmap *mask;

	gtk_widget_realize(MainWindow);
	switch (button_type)
	{
		case BUTTON_OK:
			Label = gtk_label_new(_(" OK "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,ok_xpm);
			break;

		case BUTTON_YES:
			Label = gtk_label_new(_(" Yes "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,yes_xpm);
			break;

		case BUTTON_NO:
			Label = gtk_label_new(_(" No "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,no_xpm);
			break;

		case BUTTON_APPLY:
			Label = gtk_label_new(_(" Apply "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,apply_xpm);
			break;

		case BUTTON_SAVE:
			Label = gtk_label_new(_(" Save "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,save_xpm);
			break;

		case BUTTON_CANCEL:
			Label = gtk_label_new(_(" Cancel "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,cancel_xpm);
			break;

		case BUTTON_CLOSE:
			Label = gtk_label_new(_(" Close "));
			pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,close_xpm);
			break;

		default:
			Button = gtk_button_new_with_label("Unknown button");
			return Button;
			break;
	}

	/* Create Icon */
	PixmapIcon = gtk_pixmap_new(pixmap,mask);
	gdk_pixmap_unref(pixmap);
	gdk_pixmap_unref(mask);

	Button = gtk_button_new();
	HBox = gtk_hbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(Button),HBox);
	/* Add items in button */
	gtk_container_add(GTK_CONTAINER(HBox),PixmapIcon);
	gtk_container_add(GTK_CONTAINER(HBox),Label);
	/* Alignment of items */
	gtk_misc_set_alignment(GTK_MISC(PixmapIcon),1,0.5);
	gtk_misc_set_alignment(GTK_MISC(Label),0,0.5);

	return Button;
}



/*
 * Return a widget with a pixmap
 */
GtkWidget *Create_Pixmap_Icon (gchar **pixmap_name)
{
	GtkWidget *PixmapIcon;
	GdkPixmap *pixmap;
	GdkBitmap *mask;

	gtk_widget_realize(MainWindow);
	pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,pixmap_name);
	PixmapIcon = gtk_pixmap_new(pixmap,mask);
	gdk_pixmap_unref(pixmap);
	gdk_pixmap_unref(mask);

	return PixmapIcon;
}



/*
 * Return a button with an icon and a label
 */
GtkWidget *Create_Button_With_Icon_And_Label (gchar **pixmap_name, gchar *label)
{
	GtkWidget *Button;
	GtkWidget *HBox;
	GtkWidget *Label;
	GtkWidget *PixmapIcon;
	GdkPixmap *pixmap;
	GdkBitmap *mask;


	Button = gtk_button_new();
	HBox = gtk_hbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(Button),HBox);

	/* Add a pixmap if not null */
	if (pixmap_name != NULL)
	{
		gtk_widget_realize(MainWindow);
		pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,pixmap_name);
		PixmapIcon = gtk_pixmap_new(pixmap,mask);
		gdk_pixmap_unref(pixmap);
		gdk_pixmap_unref(mask);
		gtk_container_add(GTK_CONTAINER(HBox),PixmapIcon);
	}

	/* Add a label if not null */
	if (label != NULL)
	{
		Label = gtk_label_new(label);
		gtk_container_add(GTK_CONTAINER(HBox),Label);
	}	

	/* Display a warning message if the both parameters are NULL */
	if (pixmap_name==NULL && label==NULL)
		g_warning("Empty button created 'adr=%x' (no icon and no label)!",(gint)Button);

	return Button;
}




/*
 * Attach an history list to a combobox
 */
void Add_To_Combo_Box_History(GtkObject *combobox)
{
    GList *list, *tmplist;
    
    if (!GTK_IS_COMBO(combobox) || strlen(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combobox)->entry)))==0)
        return;

    tmplist = list = gtk_object_get_data(GTK_OBJECT(combobox),"History");
    list = Add_String_To_Glist(list,gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(combobox)->entry),0,-1)); // PS: gtk_editable... allocate memory
    if (list)
    	gtk_combo_set_popdown_strings(GTK_COMBO(combobox),list);
    else
        gtk_list_clear_items(GTK_LIST(GTK_COMBO(combobox)->list),0,-1);

    /* Attach data only if there was no data attached */
    if (!tmplist)
        gtk_object_set_data(GTK_OBJECT(combobox),"History",list);
}


/*
 * Add the 'string' passed in parameter to the 'list'.
 * If this string already exists into list, it doesn't add it.
 */
GList *Add_String_To_Glist (GList *list, gchar *filename)
{
    GList *tmp_list;
    gint HISTORY_MAX_LENGTH = 15;

    if (!filename || strlen(filename)<=0 )
        return list;

    /* We search and remove duplicated items (matching with filename) */    
    tmp_list = list = g_list_first(list);
    while (tmp_list)
    {
        if ( strcmp(tmp_list->data,filename)==0 )
        {
            list = g_list_remove(list,tmp_list->data);
            tmp_list = g_list_first(list);
            continue;
        }
        if (tmp_list->next == NULL) break;
        tmp_list = g_list_next(tmp_list);
    }

    /* We add the filename to the beginning of the list */
    list = g_list_prepend(list,filename);
    
    /* Cut end of list if length exceeds HISTORY_MAX_LENGTH items */
    if (g_list_length(list)>HISTORY_MAX_LENGTH)
    {
        tmp_list = g_list_nth(list,HISTORY_MAX_LENGTH);
        tmp_list->prev->next = NULL;
        tmp_list->prev = NULL;
        g_list_free(tmp_list);
    }

    return list;
}



/*
 * Sort the list of MP3 ascending by filename.
 */
int Comp_Path_Ascending (MP3_File *file1,MP3_File *file2)
{
	return strcmp((file1->file_new==NULL)?file1->file:file1->file_new,
	              (file2->file_new==NULL)?file2->file:file2->file_new);
}
void Sort_Mp3list_Ascending (void)
{
    if (Mp3FileList == NULL) return;
	
	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

    Mp3FileList = g_list_first(Mp3FileList);
	ListIndex = 1;
    Mp3FileList = g_list_sort(Mp3FileList,(GCompareFunc)Comp_Path_Ascending);
    
    /* Reload files in browser list */
    Browser_List_Load_Files();

	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);
    Update_Command_Buttons_Sensivity();
}

/*
 * Sort the list of MP3 ascending by filename.
 */
int Comp_Path_Descending (MP3_File *file1,MP3_File *file2)
{
	return strcmp((file2->file_new==NULL)?file2->file:file2->file_new,
	              (file1->file_new==NULL)?file1->file:file1->file_new);
}
void Sort_Mp3list_Descending (void)
{
    if (Mp3FileList == NULL) return;

	Id3tag_Save_Displayed_Tag((MP3_File *)Mp3FileList->data);

    Mp3FileList = g_list_first(Mp3FileList);
	ListIndex = 1;
    Mp3FileList = g_list_sort(Mp3FileList,(GCompareFunc)Comp_Path_Descending);

    /* Reload files in browser list */
    Browser_List_Load_Files();

	Display_All_Mp3_Infos((MP3_File *)Mp3FileList->data);
    Update_Command_Buttons_Sensivity();
}



/*
 * Event attached to an entry to disable an other widget (for example: a button)
 * when the entry is empty
 */
void Entry_Changed_Disable_Object(GtkObject *widget_to_disable, GtkEditable *source_widget)
{
    gchar *text = NULL;
    
    if (!widget_to_disable || !source_widget) return;
    
    text = gtk_editable_get_chars(GTK_EDITABLE(source_widget),0,-1);
    if (!text || strlen(text)<1)
        gtk_widget_set_sensitive(GTK_WIDGET(widget_to_disable),FALSE);
    else
        gtk_widget_set_sensitive(GTK_WIDGET(widget_to_disable),TRUE);
        
    if (text) g_free(text);
}    



/*
 * Set the color of tooltips to yellow
 */
void Set_Tips_Color(GtkTooltips *tips)
{
	GtkStyle *style;
	GdkColor YELLOW = {0, 0xffff, 0xffff, 0xcccc};

	/* Tips coloration */
	gtk_tooltips_force_window(tips);
	style = gtk_style_copy(gtk_widget_get_style(tips->tip_window));
	style->bg[GTK_STATE_NORMAL] = YELLOW;
	gtk_widget_set_style(tips->tip_window,style);
}



/*
 * To insert only digits in an entry. If the text contains only digits: returns it,
 * else only first digits.
 */
#include <ctype.h>
void Insert_Only_Digit (GtkEditable *editable, const gchar *text, gint length,
			gint *position, gpointer data)
{
	gint i;
	gchar *result = g_new (gchar, length);

	for (i=0; i<length; i++)
		result[i] = isdigit(text[i])?text[i]:(gchar)NULL;
	gtk_signal_handler_block_by_func(GTK_OBJECT(editable),GTK_SIGNAL_FUNC(Insert_Only_Digit),data);
	gtk_editable_insert_text (editable, result, length, position);
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(editable),GTK_SIGNAL_FUNC(Insert_Only_Digit),data);
	gtk_signal_emit_stop_by_name(GTK_OBJECT(editable),"insert_text");
	g_free (result);
}



/*
 * Parse and auto complete date entry if you don't type the 4 digits.
 */
#include <time.h>
#include <stdlib.h>
#include <math.h>
void Parse_Date (void)
{
	gchar *year;
	gchar *tmp;
	gchar current_year[5];
	time_t t;
	struct tm t0;

	if (!DATE_AUTO_COMPLETION) return;

	/* Get the info entered by user */
	year = gtk_entry_get_text(GTK_ENTRY(YearEntry));

	if ( strcmp(year,"")!=0 && strlen(year)<4 )
	{
		t = time(NULL);
		/* Get the actual date */
		memcpy(&t0, localtime(&t), sizeof(struct tm));
		/* Put the current year in 'current_year' tab */
		sprintf(current_year,"%d",1900+t0.tm_year);

		tmp = &current_year[4-strlen(year)];
		if ( atoi(year) <= atoi(tmp) )
		{
			sprintf(current_year,"%d",atoi(current_year)-atoi(tmp));
			gtk_entry_set_text(GTK_ENTRY(YearEntry),
					           g_strdup_printf("%d",atoi(current_year)+atoi(year)));

		}else
		{
			sprintf(current_year,"%d",atoi(current_year)-atoi(tmp)-(gint)pow(10,strlen(year)));
			gtk_entry_set_text(GTK_ENTRY(YearEntry),
					           g_strdup_printf("%d",atoi(current_year)+atoi(year)));
		}
	}
}




/*
 * Load the character translation table if needed and possible
 */
void Init_Character_Translation_Table (void)
{
	if (USE_TRANSLATION_TABLE)
	{
		if (load_trans_table(TRANSLATION_TABLE_PATH)==0)
		{
			USE_TRANSLATION_TABLE = 0;
			g_print(_("Character Translation Table file invalid!\n"));
		}else
		{
			g_print(_("\nCharacter Translation Table file (%s) loaded...\n"),
				g_basename(TRANSLATION_TABLE_PATH));
		}
	}
}



/*
 * Change mouse cursor
 */
void Init_Mouse_Cursor (void)
{
	MouseCursor = NULL;
}
void Destroy_Mouse_Cursor (void)
{
	if (MouseCursor)
	{
	        gdk_cursor_destroy(MouseCursor);
		MouseCursor = NULL;
	}
}
void Set_Busy_Cursor (void)
{
	/* If still built, destroy it to avoid memory leak */
	Destroy_Mouse_Cursor();
	/* Create the new cursor */
	MouseCursor = gdk_cursor_new (GDK_WATCH);
	gdk_window_set_cursor(MainWindow->window,MouseCursor);
}
void Set_Unbusy_Cursor (void)
{
	/* Back to standard cursor */
	gdk_window_set_cursor(MainWindow->window,NULL);
	Destroy_Mouse_Cursor();
}




/*******************************************
* Functions for character set translation *
*******************************************/

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
/* 
 * Function for translation from source to program
 * Example: russian -> cp1251 to koi8
 */
void
serv2user (unsigned char *s)
{
    if (s != NULL)
	    for (; *s; ++s)
		    *s = trans_serv2user[*s];
}

/*
 * Function for translation from program to source
 * Example: russian -> koi8 to cp1251
 */
void
user2serv (unsigned char *s)
{
    if (s != NULL)
	    for (; *s; ++s)
		    *s = trans_user2serv[*s];
}

/*
 * Function receives full path to translation file (use a ircII style 
 * translation file) and generates tables.
 * Return 1 if succeed, 0 if fails. When it fails, it generates 
 * valid tables, which do no translation.
 */
int
load_trans_table (char *full_path)
{
	int tf, i, st, val = 0, t;
	char r;

	if ((tf = open (full_path, O_RDONLY)) != -1)
	{
		st = 0;
		i = 0;
		t = 0;
		while (read (tf, &r, 1) == 1)
		{
			switch (st)
			{
			case 0:			  /* nothing yet... */
				if (r == '0')
					st = 1;
				break;
			case 1:
				if (r == 'x')
					st = 2;
				else
					st = 0;
				break;
			case 2:
				if (r <= '9' && r >= '0')
					val = 16 * (r - '0');
				else if (r <= 'f' && r >= 'a')
					val = 16 * (r - 'a' + 10);
				else if (r <= 'F' && r >= 'A')
					val = 16 * (r - 'A' + 10);
				else
				{
					st = 0;
					break;
				}
				st = 3;
				break;
			case 3:
				if (r <= '9' && r >= '0')
					val += r - '0';
				else if (r <= 'f' && r >= 'a')
					val += r - 'a' + 10;
				else if (r <= 'F' && r >= 'A')
					val += r - 'A' + 10;
				else
				{
					st = 0;
					break;
				}
				st = 0;
				if (t == 0)
					trans_serv2user[i++] = val;
				else
					trans_user2serv[i++] = val;
				if (i == 256)
				{
					if (t == 1)
					{
						close (tf);
						return 1;
					}
					t = 1;
					i = 0;
				}
				break;
			default:	        /* impossible */
				close (tf);
				return 0;
			}
		}
		close (tf);
	}
	for (tf = 0; tf < 256; ++tf)
	{
		trans_user2serv[tf] = tf;
		trans_serv2user[tf] = tf;
	}
	return 0;
}

