/* undo.c - 2000/06/07 */
/*
 *  EasyTAG - Viewer/Editor for MP3 Tags
 *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>

#include "undo.h"
#include "easytag.h"
#include "header.h"
#include "id3tag.h"
#include "bar.h"
#include "msgbox.h"
#include "mylocale.h"


/***************
 * Declaration *
 ***************/


/**************
 * Prototypes *
 **************/
gboolean Compare_Tag_Between_Mp3list_And_Snap_Shot (MP3_File *mp3file);
gboolean Compare_Filename_Between_Mp3list_And_Snap_Shot (MP3_File *mp3file);
void Undo_Add_Data_To_List (MP3_File *mp3file);
void Undo_Remove_Data_From_List (void);


/*************
 * Functions *
 *************/

/*
 * Take a snapshot to keep initials informations. Taken when the file informations
 * have been displayed and not modified yet.
 */
void Undo_Get_Snap_Shot (MP3_File *mp3file)
{
	/* Free current data into "snap shot" */
	Undo_Free_Snap_Shot_Data();

	UndoSnapShot = g_malloc0(sizeof(MP3_File));
	UndoSnapShot->file = g_strdup(mp3file->file);
	if (mp3file->file_new)
		UndoSnapShot->file_new = g_strdup(mp3file->file_new);
	else
		UndoSnapShot->file_new = NULL;

	if (mp3file->id3.title)
		UndoSnapShot->id3.title   = g_strdup(mp3file->id3.title);
	if (mp3file->id3.artist)
		UndoSnapShot->id3.artist  = g_strdup(mp3file->id3.artist);
	if (mp3file->id3.album)
		UndoSnapShot->id3.album   = g_strdup(mp3file->id3.album);
	if (mp3file->id3.year)
		UndoSnapShot->id3.year    = g_strdup(mp3file->id3.year);
	if (mp3file->id3.track)
		UndoSnapShot->id3.track   = g_strdup(mp3file->id3.track);
	if (mp3file->id3.genre)
		UndoSnapShot->id3.genre   = g_strdup(mp3file->id3.genre);
	if (mp3file->id3.comment)
		UndoSnapShot->id3.comment = g_strdup(mp3file->id3.comment);

	/* For debugging */
	//Undo_Print_Snap_Shot(__FILE__,__LINE__,TRUE);
}

void Undo_Free_Snap_Shot_Data (void)
{
	if (UndoSnapShot!=NULL)
	{
		if (UndoSnapShot->file)        g_free(UndoSnapShot->file);
		if (UndoSnapShot->file_new)    g_free(UndoSnapShot->file_new);
		if (UndoSnapShot->id3.title)   g_free(UndoSnapShot->id3.title);
		if (UndoSnapShot->id3.artist)  g_free(UndoSnapShot->id3.artist);
		if (UndoSnapShot->id3.album)   g_free(UndoSnapShot->id3.album);
		if (UndoSnapShot->id3.year)    g_free(UndoSnapShot->id3.year);
		if (UndoSnapShot->id3.track)   g_free(UndoSnapShot->id3.track);
		if (UndoSnapShot->id3.genre)   g_free(UndoSnapShot->id3.genre);
		if (UndoSnapShot->id3.comment) g_free(UndoSnapShot->id3.comment);
		g_free(UndoSnapShot);
		UndoSnapShot = NULL;
	}
}



/*
 * Add a new item (the UndoSnapShot) at the end of the undo list 'UndoDataList'
 * if changes are detected.
 */
gboolean Undo_Add_Data (MP3_File *mp3file)
{
	if (   Compare_Tag_Between_Mp3list_And_Snap_Shot(mp3file)      == TRUE 
	    || Compare_Filename_Between_Mp3list_And_Snap_Shot(mp3file) == TRUE )
	{
		/* Data changed  (file renamed or tag modified */
		Undo_Add_Data_To_List(UndoSnapShot);
		return TRUE;
	}
	return FALSE;
}

void Undo_Add_Data_To_List (MP3_File *mp3file)
{
	MP3_File *UndoData;

	if (!UndoSnapShot) return;

	/* Add to undo list */
	UndoData = g_malloc0(sizeof(MP3_File));
	UndoData->file = g_strdup(UndoSnapShot->file);
	if (UndoSnapShot->file_new)
		UndoData->file_new = g_strdup(UndoSnapShot->file_new);
	else
		UndoData->file_new = NULL;

	if (UndoSnapShot->id3.title)   UndoData->id3.title   = g_strdup(UndoSnapShot->id3.title);
	else                           UndoData->id3.title   = NULL;
	if (UndoSnapShot->id3.artist)  UndoData->id3.artist  = g_strdup(UndoSnapShot->id3.artist);
	else                           UndoData->id3.artist  = NULL;
	if (UndoSnapShot->id3.album)   UndoData->id3.album   = g_strdup(UndoSnapShot->id3.album);
	else                           UndoData->id3.album   = NULL;
	if (UndoSnapShot->id3.year)    UndoData->id3.year    = g_strdup(UndoSnapShot->id3.year);
	else                           UndoData->id3.year    = NULL;
	if (UndoSnapShot->id3.track)   UndoData->id3.track   = g_strdup(UndoSnapShot->id3.track);
	else                           UndoData->id3.track   = NULL;
	if (UndoSnapShot->id3.genre)   UndoData->id3.genre   = g_strdup(UndoSnapShot->id3.genre);
	else                           UndoData->id3.genre   = NULL;
	if (UndoSnapShot->id3.comment) UndoData->id3.comment = g_strdup(UndoSnapShot->id3.comment);
	else                           UndoData->id3.comment = NULL;

	/* Add this item at the end of the undo list */
	UndoDataList = g_list_last(UndoDataList);
	UndoDataList = g_list_append(UndoDataList,UndoData);

	g_print(_("Added to undo list: %s\n"),g_basename(UndoData->file));

	/* For debugging */
	//Undo_Print_Undo_Data_List (__FILE__,__LINE__,TRUE);
}

/*
 * Check if tag infos entered by user (at this stage, there are saved in mp3 list) match 
 * (i.e. no change) or not with infos contained in the snapshot. If match: returns 0, else 1.
 * At the first dismatching info, it exits from function.
 */
gboolean Compare_Tag_Between_Mp3list_And_Snap_Shot (MP3_File *mp3file)
{
	if (!mp3file || !UndoSnapShot)
		return FALSE;

	/* Changes into tag */
	if (   ( mp3file->id3.title && !UndoSnapShot->id3.title && strlen(mp3file->id3.title)>0 )
	    || (!mp3file->id3.title &&  UndoSnapShot->id3.title && strlen(UndoSnapShot->id3.title)>0 )
	    || ( mp3file->id3.title &&  UndoSnapShot->id3.title 
		&& strlen(mp3file->id3.title)>0 && strlen(UndoSnapShot->id3.title)>0
		&& strcmp(mp3file->id3.title,UndoSnapShot->id3.title)!=0) )
		goto Changed;

	if (   ( mp3file->id3.artist && !UndoSnapShot->id3.artist && strlen(mp3file->id3.artist)>0 )
	    || (!mp3file->id3.artist &&  UndoSnapShot->id3.artist && strlen(UndoSnapShot->id3.artist)>0 )
	    || ( mp3file->id3.artist &&  UndoSnapShot->id3.artist 
		&& strlen(mp3file->id3.artist)>0 && strlen(UndoSnapShot->id3.artist)>0
		&& strcmp(mp3file->id3.artist,UndoSnapShot->id3.artist)!=0) )
		goto Changed;

	if (   ( mp3file->id3.album && !UndoSnapShot->id3.album && strlen(mp3file->id3.album)>0 )
	    || (!mp3file->id3.album &&  UndoSnapShot->id3.album && strlen(UndoSnapShot->id3.album)>0 )
	    || ( mp3file->id3.album &&  UndoSnapShot->id3.album
		&& strlen(mp3file->id3.album)>0 && strlen(UndoSnapShot->id3.album)>0
		&& strcmp(mp3file->id3.album,UndoSnapShot->id3.album)!=0) )
		goto Changed;

	if (   ( mp3file->id3.year && !UndoSnapShot->id3.year && strlen(mp3file->id3.year)>0 )
	    || (!mp3file->id3.year &&  UndoSnapShot->id3.year && strlen(UndoSnapShot->id3.year)>0 )
	    || ( mp3file->id3.year &&  UndoSnapShot->id3.year
		&& strlen(mp3file->id3.year)>0 && strlen(UndoSnapShot->id3.year)>0
		&& strcmp(mp3file->id3.year,UndoSnapShot->id3.year)!=0) )
		goto Changed;

	if (   ( mp3file->id3.track && !UndoSnapShot->id3.track && strlen(mp3file->id3.track)>0 )
	    || (!mp3file->id3.track &&  UndoSnapShot->id3.track && strlen(UndoSnapShot->id3.track)>0 )
	    || ( mp3file->id3.track &&  UndoSnapShot->id3.track 
		 && strlen(mp3file->id3.track)>0 && strlen(UndoSnapShot->id3.track)>0
		 && strcmp(mp3file->id3.track,UndoSnapShot->id3.track)!=0) )
		goto Changed;

	if (   ( mp3file->id3.genre && !UndoSnapShot->id3.genre && strlen(mp3file->id3.genre)>0 )
	    || (!mp3file->id3.genre &&  UndoSnapShot->id3.genre && strlen(UndoSnapShot->id3.genre)>0 )
	    || ( mp3file->id3.genre &&  UndoSnapShot->id3.genre 
		&& strlen(mp3file->id3.genre)>0 && strlen(UndoSnapShot->id3.genre)>0
		&& strcmp(mp3file->id3.genre,UndoSnapShot->id3.genre)!=0) )
		goto Changed;

	if (   ( mp3file->id3.comment && !UndoSnapShot->id3.comment && strlen(mp3file->id3.comment)>0 )
	    || (!mp3file->id3.comment &&  UndoSnapShot->id3.comment && strlen(UndoSnapShot->id3.comment)>0 )
	    || ( mp3file->id3.comment &&  UndoSnapShot->id3.comment 
		&& strlen(mp3file->id3.comment)>0 && strlen(UndoSnapShot->id3.comment)>0
		&& strcmp(mp3file->id3.comment,UndoSnapShot->id3.comment)!=0) )
		goto Changed;

    goto NotChanged;
    
	NotChanged:
		/* If you pass here => All match => no change */
		return FALSE;
	Changed:
		/* Changes detected */
		mp3file->changed |= TAG_CHANGED;
		return TRUE;
}

/*
 * Check if filename infos entered by user (at this stage, there are saved in mp3 list) match 
 * (i.e. no change) or not with infos contained in the snapshot. If match: returns 0, else 1.
 * At the first dismatching info, it exits from function.
 */
gboolean Compare_Filename_Between_Mp3list_And_Snap_Shot (MP3_File *mp3file)
{
	if (!mp3file || !UndoSnapShot)
		return FALSE;

	/* Filename changes */
	if (   ( mp3file->file_new && !UndoSnapShot->file_new && strlen(mp3file->file_new)>0 )
	    || (!mp3file->file_new &&  UndoSnapShot->file_new && strlen(UndoSnapShot->file_new)>0 ) 
	    || ( mp3file->file_new &&  UndoSnapShot->file_new 
		&& strlen(mp3file->file_new)>0 && strlen(UndoSnapShot->file_new)>0
		&& strcmp(mp3file->file_new,UndoSnapShot->file_new)!=0) )
		goto Changed;

    goto NotChanged;
    
	NotChanged:
		/* If you pass here => All match => no change */
		return FALSE;
	Changed:
		/* Changes detected */
		mp3file->changed |= FILENAME_CHANGED;
		return TRUE;
}


/*
 * Restore the last item of the undo list to cancel the last change
 */
gboolean Undo_Restore_Data (void)
{
	GList *undolist_save_start = NULL;
	GList *mp3list_save_position = NULL;
	gint listindex_save_value;
	GtkWidget *msgbox;
	gchar *msg;
	

	/* Get the last entry in undo list (it's the last tag changed) */
	if (!UndoDataList) return 0;
	undolist_save_start = g_list_first(UndoDataList);    // Save the address of the first item
	UndoDataList = g_list_last(UndoDataList);

    mp3list_save_position = Mp3FileList;    // Save address of the current item
    listindex_save_value = ListIndex;
    
    /* Find in the undo list the good item (same filename). The first file is the last 
     * one of the undo list, and it try to find the same in the Mp3FileList list */
	Mp3FileList = g_list_first(Mp3FileList);
	ListIndex=1;
    while ( Mp3FileList && strcmp ( ((MP3_File *)UndoDataList->data)->file,
                                    ((MP3_File *)Mp3FileList ->data)->file )!=0 )
	{
		/* Check next file */
		if (!Mp3FileList->next)
		{
			/* The undo data is corrupted */
            msg = g_strdup_printf(_("Undo error: can't find file '%s' in the undo list!\n"),
                ((MP3_File *)UndoDataList->data)->file); 
    		msgbox = msg_box_new (_("Error..."),msg,MSG_ERROR,BUTTON_OK,0);
            g_free(msg);
		    msg_box_hide_check_button(MSG_BOX(msgbox));
		    msg_box_run(MSG_BOX(msgbox));
		    gtk_widget_destroy(msgbox);

    		UndoDataList = undolist_save_start;
    		Mp3FileList = mp3list_save_position;
    		ListIndex = listindex_save_value;
			return 0; 
		}
		Mp3FileList = g_list_next(Mp3FileList);
		ListIndex++;
	}

	/* Display the tag data */
	//Id3tag_Display_Tag((MP3_File *)Mp3FileList->data);
	Header_Display((MP3_File *)Mp3FileList->data);

	/* Undo changes */
	Id3tag_Display_Tag((MP3_File *)UndoDataList->data);
	Id3tag_Save_Displayed_Tag((MP3_File *)(Mp3FileList->data));

	g_print(_("Undo restored: %s\n"),g_basename(((MP3_File *)UndoDataList->data)->file));

	/* Remove the undo item from the undo list */
	if (UndoDataList->prev != NULL)
		UndoDataList->prev->next = NULL;
	if (((MP3_File *)UndoDataList->data)->file)
		g_free(((MP3_File *)UndoDataList->data)->file);
	if (((MP3_File *)UndoDataList->data)->file_new)
	 	g_free(((MP3_File *)UndoDataList->data)->file_new);
	if (((MP3_File *)UndoDataList->data)->id3.title)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.title);
	if (((MP3_File *)UndoDataList->data)->id3.artist)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.artist);
	if (((MP3_File *)UndoDataList->data)->id3.album)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.album);
	if (((MP3_File *)UndoDataList->data)->id3.year)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.year);
	if (((MP3_File *)UndoDataList->data)->id3.track)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.track);
	if (((MP3_File *)UndoDataList->data)->id3.genre)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.genre);
	if (((MP3_File *)UndoDataList->data)->id3.comment)
	 	g_free(((MP3_File *)UndoDataList->data)->id3.comment);
	g_free(UndoDataList->data);

	if (undolist_save_start==UndoDataList)
	{
		/* It was the 1rst item, so the list is empty now */
		UndoDataList = NULL;
	}else
	{
		UndoDataList = undolist_save_start;
	}
	return 1;
}



/*
 * Free all items and data of the undo list
 */
void Undo_Free_Undo_Data_List (void)
{
	GList *list_save;	// To save address of first item
	
	if (UndoDataList == NULL) return;
	list_save = UndoDataList = g_list_first(UndoDataList);
	while (UndoDataList)
	{	
		/* Filename items */
		if (((MP3_File *)UndoDataList->data)->file != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->file );
		if (((MP3_File *)UndoDataList->data)->file_new != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->file_new );

		/* Tag items */
		if (((MP3_File *)UndoDataList->data)->id3.title != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.title );
		if (((MP3_File *)UndoDataList->data)->id3.artist != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.artist );
		if (((MP3_File *)UndoDataList->data)->id3.album != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.album );
		if (((MP3_File *)UndoDataList->data)->id3.year != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.year );
		if (((MP3_File *)UndoDataList->data)->id3.track != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.track );
		if (((MP3_File *)UndoDataList->data)->id3.genre != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.genre );
		if (((MP3_File *)UndoDataList->data)->id3.comment != NULL)
			g_free( ((MP3_File *)UndoDataList->data)->id3.comment );

		/* Free GList item data */
		g_free(UndoDataList->data);
		UndoDataList = g_list_next(UndoDataList);
	}
	UndoDataList = list_save;
	g_list_free(UndoDataList);
	UndoDataList = NULL;
}


/*
 * Send a message to the statubar with informations about the next undo data 
 * into the undo list.
 */
void Undo_Display_Last_Info (void)
{
	gchar *msg;
	guint num;

	if (UndoDataList)
	{
		UndoDataList = g_list_first(UndoDataList);
		num = g_list_length(UndoDataList);
		UndoDataList = g_list_last(UndoDataList);

		msg = g_strdup_printf(_("(#:%d) Next undo for: %s"),num,
			g_basename(((MP3_File *)UndoDataList->data)->file));
	}else
	{
		msg = g_strdup(_("No undo data!"));
	}

	Statusbar_Message(_(msg),TRUE);
	g_free(msg);
}



/*
 * This function updates the name of files after saving changes. Cause, after saving files, 
 * the name of files on hard disk and in undo list may dismatch (when renaming file).
 * And do the same thing for the snap shot.
 */
void Undo_Update_Data (MP3_File *mp3file)
{
	/*
	 * This part updates data into Undo list
	 */
    UndoDataList = g_list_first(UndoDataList);
	while (UndoDataList)
	{
		/* Search items with the same name into undo list, if file name has been changed */
		if ( strcmp(((MP3_File *)Mp3FileList->data)->file,((MP3_File *)UndoDataList->data)->file)==0
		     && ((MP3_File *)Mp3FileList->data)->file_new )
		{
			/* "file_new" part of undo data */
			if (!((MP3_File *)UndoDataList->data)->file_new)
			{
				((MP3_File *)UndoDataList->data)->file_new = 
					g_strdup(((MP3_File *)UndoDataList->data)->file);
			}

			/* then "file" part of undo data */
			if (((MP3_File *)UndoDataList->data)->file)
				g_free(((MP3_File *)UndoDataList->data)->file);
			((MP3_File *)UndoDataList->data)->file = g_strdup(((MP3_File *)Mp3FileList->data)->file_new);
		}

		if (!UndoDataList->next) break;
		UndoDataList = g_list_next(UndoDataList);
	}

	/*
	 * This part updates data into Snap Shot
	 */
	if (UndoSnapShot && strcmp(UndoSnapShot->file,((MP3_File *)Mp3FileList->data)->file)==0)
	{
		g_free(UndoSnapShot->file);
		UndoSnapShot->file = g_strdup(((MP3_File *)Mp3FileList->data)->file_new);
		if ( UndoSnapShot->file_new && strcmp(UndoSnapShot->file_new,UndoSnapShot->file)==0 )
		{
			g_free(UndoSnapShot->file_new);
			UndoSnapShot->file_new = NULL;
		}
	}

	/* For debugging */
	//Undo_Print_Undo_Data_List (__FILE__,__LINE__,TRUE);
}




/*
 * Functions for DEBUGGING ONLY
 * Undo_Print_Snap_Shot        => show what contains the snapshot
 * Undo_Print_Undo_Data_List   => show what contains the list of undo
 */
void Undo_Print_Snap_Shot (gchar *file ,gint line, gboolean show_tag)
{
	g_print("\n#### Snapshot from %s:%d - start ####\n",file,line);
	if (!UndoSnapShot)
		g_print("No Snapshot\n");
	else
	{
		g_print("# file       : '%s'\n",UndoSnapShot->file);
		g_print("# file_new   : '%s'\n",UndoSnapShot->file_new);
		if (show_tag)
		{
			g_print("# id3.title  : '%s'\n",UndoSnapShot->id3.title);
			g_print("# id3.artist : '%s'\n",UndoSnapShot->id3.artist);
			g_print("# id3.album  : '%s'\n",UndoSnapShot->id3.album);
			g_print("# id3.year   : '%s'\n",UndoSnapShot->id3.year);
			g_print("# id3.genre  : '%s'\n",UndoSnapShot->id3.genre);
			g_print("# id3.track  : '%s'\n",UndoSnapShot->id3.track);
			g_print("# id3.comment: '%s'\n",UndoSnapShot->id3.comment);
		}
	}
	g_print("#### Snapshot from %s:%d - end   ####\n",file,line);
}
void Undo_Print_Undo_Data_List (gchar *file ,gint line, gboolean all)
{
	gint item = 1;
	GList *savelist = NULL;

	g_print("\n#### Undo list from %s:%d - start ####\n",file,line);
	/* Get the current position in list */
	savelist = UndoDataList;
	UndoDataList = g_list_first(UndoDataList);
	g_print("### length   : '%d'\n",g_list_length(UndoDataList));
	while (UndoDataList)
	{
		g_print("### item %d\n",item);
		g_print("# file       : '%s'\n",((MP3_File *)UndoDataList->data)->file);
		g_print("# file_new   : '%s'\n",((MP3_File *)UndoDataList->data)->file_new);
		if (all)
		{
			g_print("# id3.title  : '%s'\n",((MP3_File *)UndoDataList->data)->id3.title);
			g_print("# id3.artist : '%s'\n",((MP3_File *)UndoDataList->data)->id3.artist);
			g_print("# id3.album  : '%s'\n",((MP3_File *)UndoDataList->data)->id3.album);
			g_print("# id3.year   : '%s'\n",((MP3_File *)UndoDataList->data)->id3.year);
			g_print("# id3.genre  : '%s'\n",((MP3_File *)UndoDataList->data)->id3.genre);
			g_print("# id3.track  : '%s'\n",((MP3_File *)UndoDataList->data)->id3.track);
			g_print("# id3.comment: '%s'\n",((MP3_File *)UndoDataList->data)->id3.comment);
		}

		if (!UndoDataList->next) break;
		UndoDataList = g_list_next(UndoDataList);
		item++;
	}
	/* Restore the current position in list */
	UndoDataList = savelist;
	g_print("#### Undo list from %s:%d - end   ####\n",file,line);
}
